﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourQLInterpreter.Reasoning;

namespace FourQLInterpreter.Application
{
    /// <summary>
    /// http://stackoverflow.com/a/856918
    /// </summary>
    class TablePrettyPrinter
    {
        static int tableWidth = 77;

        public static void PrintLine()
        {
            Console.WriteLine(new string('-', tableWidth));
        }

        public static void PrintRow(params string[] columns)
        {
            int width = (tableWidth - columns.Length) / columns.Length;
            string row = "|";

            foreach (string column in columns)
            {
                row += AlignCentre(column, width) + "|";
            }

            Console.WriteLine(row);
        }

        public static string AlignCentre(string text, int width)
        {
            text = text.Length > width ? text.Substring(0, width - 3) + "..." : text;

            if (string.IsNullOrEmpty(text))
            {
                return new string(' ', width);
            }
            else
            {
                return text.PadRight(width - (width - text.Length) / 2).PadLeft(width);
            }
        }

        public static void PrintResult(Result result)
        {
            var headerRow = (from header in result.ColumnHeaders 
                select header.Name).ToList();

            headerRow.Add("Logic value");
            PrintLine();
            PrintRow(headerRow.ToArray());
            PrintLine();
            PrintLine();
            foreach (var resultRow in result.Rows)
            {
                var row = (from cell in resultRow.Values
                    select cell.Value.ToString()).ToList();
                row.Add(resultRow.Value.ToString());
                PrintRow(row.ToArray());
                PrintLine();
            }
        }
    }
}
