﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Structures;

namespace FourQLInterpreter.InternalModules
{
    public class MathModule : InternalModule
    {

        public MathModule()
            : base()
        {
            this.Name = "math";

            this.Relations.Add(new Relation() { Name = "lt", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "ltR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL } } });

            this.Relations.Add(new Relation() { Name = "gt", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "gtR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL } } });


            this.Relations.Add(new Relation() { Name = "le", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "leR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL } } });

            this.Relations.Add(new Relation() { Name = "ge", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "geR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL } } });

            this.Relations.Add(new Relation() { Name = "eq", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.LITERAL }, { 1, DomainType.LITERAL } } });
            this.Relations.Add(new Relation() { Name = "eqI", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "eqR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL } } });
            this.Relations.Add(new Relation() { Name = "eqS", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.STRING }, { 1, DomainType.STRING } } });
            this.Relations.Add(new Relation() { Name = "eqD", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.DATE }, { 1, DomainType.DATE } } });
            this.Relations.Add(new Relation() { Name = "eqDT", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.DATE_TIME }, { 1, DomainType.DATE_TIME } } });
            this.Relations.Add(new Relation() { Name = "eqL", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.LOGIC }, { 1, DomainType.LOGIC } } });


            // X + Y = Z 
            this.Relations.Add(new Relation() { Name = "sum", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER }, { 2, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "sumR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL }, { 2, DomainType.REAL } } });


            //  X * Y = Z 
            this.Relations.Add(new Relation() { Name = "product", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER }, { 2, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "productR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL }, { 2, DomainType.REAL } } });


            //  X / Y = Z
            this.Relations.Add(new Relation() { Name = "quotient", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER }, { 2, DomainType.REAL } } });
            this.Relations.Add(new Relation() { Name = "quotientR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL }, { 2, DomainType.REAL } } });


            //  Z = min{X, Y} 
            this.Relations.Add(new Relation() { Name = "min", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER }, { 2, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "minR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL }, { 2, DomainType.REAL } } });


            // Z = max{X, Y} 
            this.Relations.Add(new Relation() { Name = "max", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER }, { 2, DomainType.INTEGER } } });
            this.Relations.Add(new Relation() { Name = "maxR", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.REAL }, { 2, DomainType.REAL } } });


            // Z = round(X) to Y digits
            this.Relations.Add(new Relation() { Name = "round", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.REAL }, { 1, DomainType.INTEGER }, { 2, DomainType.REAL } } });
        }

        public override LogicValue GetLiteralValue(string relationName, IList<ValueHolder> values)
        {
            switch (relationName)
            {
                case "lt":
                    return values[0].valueAs<int>() < values[1].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "ltR":
                    return values[0].valueAs<double>() < values[1].valueAs<double>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "gt":
                    return values[0].valueAs<int>() > values[1].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "gtR":
                    return values[0].valueAs<double>() > values[1].valueAs<double>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "lte":
                    return values[0].valueAs<int>() <= values[1].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "lteR":
                    return values[0].valueAs<double>() <= values[1].valueAs<double>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "gte":
                    return values[0].valueAs<int>() > values[1].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "gteR":
                    return values[0].valueAs<double>() > values[1].valueAs<double>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "eq":
                case "eqI":
                case "eqR":
                case "eqS":
                case "eqD":
                case "eqDT":
                case "eqL":
                    return values[0].Value.Equals(values[1].Value) ? LogicValue.TRUE : LogicValue.FALSE;
                case "sum":
                    return values[0].valueAs<int>() + values[1].valueAs<int>() == values[2].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "sumR":
                    return (values[0].valueAs<double>() + values[1].valueAs<double>()).CompareTo(values[2].valueAs<double>()) == 0 ? LogicValue.TRUE : LogicValue.FALSE;
                case "product":
                    return values[0].valueAs<int>() * values[1].valueAs<int>() == values[2].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "productR":
                    return (values[0].valueAs<double>() * values[1].valueAs<double>()).CompareTo(values[2].valueAs<double>()) == 0 ? LogicValue.TRUE : LogicValue.FALSE;
                case "quotient":
                case "quotientR":
                    return (values[0].valueAs<double>() / values[1].valueAs<double>()).CompareTo(values[2].valueAs<double>()) == 0 ? LogicValue.TRUE : LogicValue.FALSE;
                case "min":
                    return ((values[0].valueAs<int>() < values[1].valueAs<int>()) ? values[0].valueAs<int>() : values[1].valueAs<int>()) == values[2].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "minR":
                    return ((values[0].valueAs<double>() < values[1].valueAs<double>()) ? values[0].valueAs<double>() : values[1].valueAs<double>()).CompareTo(values[2].valueAs<double>()) == 0 ? LogicValue.TRUE : LogicValue.FALSE;
                case "max":
                    return ((values[0].valueAs<int>() > values[1].valueAs<int>()) ? values[0].valueAs<int>() : values[1].valueAs<int>()) == values[2].valueAs<int>() ? LogicValue.TRUE : LogicValue.FALSE;
                case "maxR":
                    return ((values[0].valueAs<double>() > values[1].valueAs<double>()) ? values[0].valueAs<double>() : values[1].valueAs<double>()).CompareTo(values[2].valueAs<double>()) == 0 ? LogicValue.TRUE : LogicValue.FALSE;
                case "round":
                    return (Math.Round(values[0].valueAs<double>(), values[1].valueAs<int>()).Equals(values[2].valueAs<double>()))  ? LogicValue.TRUE : LogicValue.FALSE;

                default:
                    return LogicValue.UNKNOWN;
            }
        }

    }



}
