﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using CSScriptLibrary;
using FourqlParser.Structures;
using NLog;

namespace FourQLInterpreter.InternalModules
{
    /// <summary>
    /// Represents a plugin as a 4QL module
    /// </summary>
    public class PluginModuleWrapper : InternalModule
    {
        private static readonly Logger Log = LogManager.GetCurrentClassLogger();

        private IPluginModule _plugin;

        private string _file;

        public PluginModuleWrapper(string pluginFile) : base()
        {
            _file = pluginFile;
        }

        public bool Initialize()
        {
            try
            {
                Log.Trace("Loading plugin file " + _file);
                _plugin = CSScript.Evaluator.LoadFile<IPluginModule>(_file);
                this.Name = _plugin.GetModuleName();
                foreach (var relation in _plugin.GetDefinedRelations())
                {
                    this.Relations.Add(relation);
                }
                return true;
            }
            catch (Exception e)
            {
                Log.Error("Error while initializing the plugin wrapper.", e);
                throw e;
                return false;
            }
        }

        public override FourqlParser.Structures.LogicValue GetLiteralValue(string relationName, IList<FourqlParser.Structures.ValueHolder> values)
        {
            try
            {
                Log.Debug("Getting value of " + relationName + " from an external plugin " + this.Name);
                return _plugin.GetLiteralValue(relationName, values);
            }
            catch (Exception e)
            {
                Log.Error("Error while executing the external plugin. Returning the UNKNOWN value.", e);
                return LogicValue.UNKNOWN;
            }
        }
    }
}
