﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Structures;

namespace FourQLInterpreter.Reasoning.Explanation
{
    /// <summary>
    /// Explains the result of an evaluated expression
    /// </summary>
    public class Explanation
    {
        /// <summary>
        /// Contains logic value of every literal in the expression.
        /// If any value in the list is null, that means that it could not be evaluated (happens when the domain for a quantified variable is empty
        /// and thus the expression under the quantifier cannot be evaluated)
        /// </summary>
        public IList<LogicValue?> LiteralValues { get; set; }

        /// <summary>
        /// Contains the logic value of every quantifier node.
        /// If any value in the list is null, that means that it could not be evaluated.
        /// </summary>
        public IList<LogicValue?> QuantifierValues { get; set; }

        /// <summary>
        /// Contains the 'best' / 'chosen' variable of every quantifier node in the expression.
        /// If any of the values is null, that means that the domain of the quantified variable was empty so no value can be chosen.
        /// </summary>
        public IList<ValueHolder> QuantifierVariables { get; set; }  

        public Explanation()
        {
            LiteralValues = new List<LogicValue?>();
            QuantifierValues = new List<LogicValue?>();
            QuantifierVariables = new List<ValueHolder>();
        }


        public override string ToString()
        {
            StringBuilder s = new StringBuilder("LiteralValues: ");
            foreach (var v in LiteralValues)
            {
                s.Append((v != null ? v.ToString() : "null") + " |");
            }

            if (QuantifierValues.Count > 0)
            {
                s.Append("\nQuantifierValues: ");
                foreach (var v in QuantifierValues)
                {
                    s.Append((v != null ? v.ToString() : "null") + " |");
                }
            }

            if (QuantifierVariables.Count > 0)
            {
                s.Append("\nQuantifierVariables: ");
                foreach (var v in QuantifierVariables)
                {
                    s.Append((v != null ? v.ToString() : "null") + " |");
                }
            }
            return s.ToString();
        }
    }
}
