﻿using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourQLInterpreter.Reasoning
{
    class Logic
    {
        public static LogicValue Or(LogicValue lhs, LogicValue rhs)
        {
            return lhs >= rhs ? lhs : rhs;
        }

        public static LogicValue And(LogicValue lhs, LogicValue rhs)
        {
            return lhs <= rhs ? lhs : rhs;
        }

        public static LogicValue Not(LogicValue value)
        {
            switch (value)
            {
                case LogicValue.FALSE:
                    return LogicValue.TRUE;
                case LogicValue.TRUE:
                    return LogicValue.FALSE;
                default:
                    return value;
            }
        }


        public static LogicValue Then(LogicValue antecedent, LogicValue consequent)
        {
            switch (antecedent)
            {
                case LogicValue.FALSE:
                case LogicValue.UNKNOWN:
                    return LogicValue.TRUE;
                default:
                    if (consequent.Equals(LogicValue.INCONSISTENT))
                        return LogicValue.TRUE;
                    if (consequent.Equals(antecedent))
                        return LogicValue.TRUE;
                    return LogicValue.FALSE;
            }
        }
    }

    class LogicPrim : Logic
    {
        //disjunction w.r.t. ordering u < f < t < i
       public static LogicValue OrPrim(LogicValue lhs, LogicValue rhs)
        {
            return ToPrim(lhs) >= ToPrim(rhs) ? lhs : rhs;
        }

       public static LogicValue Max()
       {
           return LogicValue.INCONSISTENT;
       }

       public static LogicValue Min()
       {
           return LogicValue.UNKNOWN;
       }


        private static LogicValuePrim ToPrim(LogicValue value)
        {
            switch (value)
            {
                case LogicValue.FALSE:
                    return LogicValuePrim.False;
                case LogicValue.INCONSISTENT:
                    return LogicValuePrim.Inconsistent;
                case LogicValue.TRUE:
                    return LogicValuePrim.True;
                default:
                    return LogicValuePrim.Unknown;
            }
        }

        private enum LogicValuePrim : int
        {
            Unknown,
            False,
            True,
            Inconsistent
        }
    }

}
