﻿using System;
using FourqlParser.Structures;
using FourQLInterpreter.Application;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    [TestClass]
    public class ApplicationTest
    {
        [TestMethod]
        public void TestMath()
        {
            var app = new Application();
            app.ParseInput("load ../../Resources/mathTest.4ql");

            var result = app.InterpreterInstance.Query("m.int(X) and m.int(Y) and math.sum(X, Y, 6)");
            Console.WriteLine(result);
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                if (m["X"].Equals(2) && m["Y"].Equals(4))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(4) && m["Y"].Equals(2))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(3) && m["Y"].Equals(3))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else
                    Assert.AreEqual(LogicValue.FALSE, m["logic_value"]);
            }

            result = app.InterpreterInstance.Query("m.int(X) and m.int(Y) and math.product(X, Y, 6)");
      

            Console.WriteLine(result);
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                if (m["X"].Equals(1) && m["Y"].Equals(6))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(6) && m["Y"].Equals(1))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(3) && m["Y"].Equals(2))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(2) && m["Y"].Equals(3))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else
                    Assert.AreEqual(LogicValue.FALSE, m["logic_value"]);
            }
        }

        [TestMethod]
        public void TestClear()
        {
            var app = new Application();
            app.ParseInput("load ../../Resources/mathTest.4ql");
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("math"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("contains"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("m"));

            app.InterpreterInstance.Query("m.int(X) and m.int(Y) and math.product(X, Y, 6)");
      
            app.ParseInput("clear");

            app.ParseInput("load ../../Resources/containsTest.4ql");
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("math"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("contains"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("m"));
            app.InterpreterInstance.Query("m.str(X) and m.str(Y) and contains.contains(X, Y)");
      
        }

        [TestMethod]
        public void TestLoadNoClear()
        {
            var app = new Application();
            app.ParseInput("load ../../Resources/mathTest.4ql");
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("math"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("contains"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("m"));
            app.InterpreterInstance.Query("m.int(X) and m.int(Y) and math.product(X, Y, 6)");

            app.ParseInput("load ../../Resources/containsTest2.4ql");
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("math"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("contains"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("m"));
            Assert.IsTrue(app.InterpreterInstance.Modules.ContainsKey("m2"));
            app.InterpreterInstance.Query("m.int(X) and m.int(Y) and math.product(X, Y, 6)");
            app.InterpreterInstance.Query("m.str(X) and m.str(Y) and contains.contains(X, Y)");
        }

        [TestMethod]
        public void TestGetHistory()
        {

            var app = new Application();
            app.InterpreterInstance.LoadModules("../../Resources/hierarchy.4ql");
        }
    }
}
