﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Parsers;
using FourqlParser.Structures;
using FourQLInterpreter.Reasoning;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    public class BaseModuleTests
    {
        protected string GetTestContent(string folder, string name)
        {
            return ParserDispatcher.AsFile("../../../4QLInterpreterTest/Resources/" + (folder != null ? folder + "/" : "") + name);
        }

        protected string GetTestContent(string name)
        {
            return GetTestContent(null, name);
        }

        /// <summary>
        /// Loads test modules from file.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        protected IDictionary<string, Module> LoadModules(string fileName)
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent(fileName), errors, null);

            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);

            return prog.Modules.ToDictionary(module => module.Name);
        }

        protected IDictionary<string, Module> LoadModules(string fileName, ICollection<Module> extraModules)
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent(fileName), errors, null);

            Assert.IsTrue(errors.ParserErrors.Count == 0);

            var prog = ParserDispatcher.toProgram(progContext);

            foreach (var module in extraModules)
            {
                prog.Modules.Add(module);
            }
            
            ParserDispatcher.checkProgram(prog);

            return prog.Modules.ToDictionary(module => module.Name);
        }
       

        protected Evaluator CreateEvaluator(ICollection<Module> moduleCollection)
        {
            IDictionary<string, ModuleDataWrapper> modules = new Dictionary<string, ModuleDataWrapper>();
            foreach (var module in moduleCollection)
            {
                var wrapper = ModuleDataWrapperFactory.CreateWrapper(module);
                wrapper.Initialize();
                modules[module.Name] = wrapper;
            }
            var evaluator = new Evaluator(modules);
            return evaluator;
        }

        protected Expression CreateExpression(string s, IDictionary<string, Module> modules, string moduleName)
        {
            var errors = new ParserDispatcher.Errors();
            var context = ParserDispatcher.parseExpr(s, errors);
            var expr = ParserDispatcher.toExpr(context);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            ParserDispatcher.checkExpr(expr, modules.Values, modules[moduleName]);
            return expr;
        }
    }
}
