﻿using System;
using System.Linq;
using FourqlParser.Structures;
using FourQLInterpreter.WSM;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    [TestClass]
    public class DomainsTest :BaseModuleTests
    {
        [TestMethod]
        public void TestDomainImports()
        {
            var modules = LoadModules("domainTests/domainImports.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            var wsm = new WsmEvaluator(modules["s"], evaluator, true);
            wsm.ConstructWsm(); 
            wsm = new WsmEvaluator(modules["n"], evaluator, true);
            wsm.ConstructWsm(); 
            wsm = new WsmEvaluator(modules["m"], evaluator, true);
            wsm.ConstructWsm();

            var mdom1 = evaluator.ModulesData["m"].DomainDict["dom1"];
            Assert.AreEqual(4, mdom1.Count);
            Assert.IsTrue(mdom1.Contains(new ValueHolder("a", DomainType.LITERAL)));
            Assert.IsTrue(mdom1.Contains(new ValueHolder("b", DomainType.LITERAL)));
            Assert.IsTrue(mdom1.Contains(new ValueHolder("c", DomainType.LITERAL)));
            Assert.IsTrue(mdom1.Contains(new ValueHolder("d", DomainType.LITERAL)));

        }

        [TestMethod]
        public void TestDomainsAreStatic()
        {
            var modules = LoadModules("domainTests/domainsAreStatic.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            var wsm = new WsmEvaluator(modules["n"], evaluator, true);
            wsm.ConstructWsm();
            wsm = new WsmEvaluator(modules["m"], evaluator, true);
            wsm.ConstructWsm();

            var mdom1 = evaluator.ModulesData["m"].DomainDict["dom1"];
            Assert.AreEqual(2, mdom1.Count);
            Assert.IsTrue(mdom1.Contains(new ValueHolder("a", DomainType.LITERAL)));
            Assert.IsTrue(mdom1.Contains(new ValueHolder("b", DomainType.LITERAL)));
            Assert.IsFalse(mdom1.Contains(new ValueHolder("c", DomainType.LITERAL)));

            var expression = CreateExpression("rel(X)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            Assert.AreEqual(2, result.Rows.Count());
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                if (m["X"].Equals("a"))
                    Assert.AreEqual(LogicValue.UNKNOWN, m["logic_value"]);
                if (m["X"].Equals("b"))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
            }

        }

        [TestMethod]
        public void TestDomainsAndQuantifiers()
        {
            var modules = LoadModules("domainTests/domainQuantifiers.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            var wsm = new WsmEvaluator(modules["main"], evaluator, true);
            wsm.ConstructWsm();

            var expression = CreateExpression("EXISTS X:dom (rel2(X))", modules, "main");
            var result = evaluator.Evaluate(expression, modules["main"]);
            Console.WriteLine(result);
            Assert.AreEqual(1, result.Rows.Count());
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                Assert.AreEqual(LogicValue.FALSE, m["logic_value"]);
            }

        }

    }
}
