﻿using System;
using System.Linq;
using FourqlParser.Structures;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    [TestClass]
    public class ExplanationTest : BaseModuleTests
    {
        [TestMethod]
        public void TestAnd()
        {
            var modules = LoadModules("testModuleNoRules.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            evaluator.Explain = true;
            var expression = CreateExpression("r(a) and r(a) and t(b)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            Assert.AreEqual(1, result.Rows.Count());
            var explanation = result.Rows.First().RowExplanation;
            Assert.AreEqual(LogicValue.TRUE, explanation.LiteralValues[0]);
            Assert.AreEqual(LogicValue.TRUE, explanation.LiteralValues[1]);
            Assert.AreEqual(LogicValue.INCONSISTENT, explanation.LiteralValues[2]);
        }

        [TestMethod]
        public void TestExists()
        {
            var modules = LoadModules("testModuleNoRules.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            evaluator.Explain = true;
           
            var expression = CreateExpression("EXISTS Y:dom (not r(Y) and t(Y)) and EXISTS X:dom n.r(X)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            Assert.AreEqual(1, result.Rows.Count());

            Assert.AreEqual(LogicValue.INCONSISTENT, result.Rows.First().Value);

            var explanation = result.Rows.First().RowExplanation;

            Assert.AreEqual(LogicValue.FALSE, explanation.LiteralValues[0]);
            Assert.AreEqual(LogicValue.INCONSISTENT, explanation.LiteralValues[1]);
            Assert.AreEqual(LogicValue.TRUE, explanation.LiteralValues[2]);

            Assert.AreEqual(LogicValue.INCONSISTENT, explanation.QuantifierValues[0]);
            Assert.AreEqual(LogicValue.TRUE, explanation.QuantifierValues[1]);

            Assert.AreEqual("b", explanation.QuantifierVariables[0].Value);
            Assert.AreEqual("a", explanation.QuantifierVariables[1].Value);
        }

        [TestMethod]
        public void TestExistsEmptyDom()
        {
            var modules = LoadModules("testModuleNoRules.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            evaluator.Explain = true;

            var expression = CreateExpression("n.(EXISTS Y:dom (not r(Y) and s(Y))) and EXISTS X:dom r(X)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            Assert.AreEqual(1, result.Rows.Count());

            Assert.AreEqual(LogicValue.FALSE, result.Rows.First().Value);

            var explanation = result.Rows.First().RowExplanation;

            Assert.AreEqual(null, explanation.LiteralValues[0]);
            Assert.AreEqual(null, explanation.LiteralValues[1]);
            Assert.AreEqual(LogicValue.TRUE, explanation.LiteralValues[2]);

            Assert.AreEqual(LogicValue.FALSE, explanation.QuantifierValues[0]);
            Assert.AreEqual(LogicValue.TRUE, explanation.QuantifierValues[1]);

            Assert.AreEqual(null, explanation.QuantifierVariables[0]);
            Assert.AreEqual("a", explanation.QuantifierVariables[1].Value);
        }

        [TestMethod]
        public void TestAll()
        {
            var modules = LoadModules("testModuleNoRules.4ql");
            var evaluator = CreateEvaluator(modules.Values);
            evaluator.Explain = true;
            var expression = CreateExpression("ALL Y:dom r(Y) or n.ALL Z:dom r(Z) ", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            Assert.AreEqual(1, result.Rows.Count());

            Assert.AreEqual(LogicValue.TRUE, result.Rows.First().Value);

            var explanation = result.Rows.First().RowExplanation;

            Assert.AreEqual(LogicValue.FALSE, explanation.LiteralValues[0]);
            Assert.AreEqual(null, explanation.LiteralValues[1]);
           
            Assert.AreEqual(LogicValue.FALSE, explanation.QuantifierValues[0]);
            Assert.AreEqual(LogicValue.TRUE, explanation.QuantifierValues[1]);

            Assert.AreEqual("b", explanation.QuantifierVariables[0].Value);
            Assert.AreEqual(null, explanation.QuantifierVariables[1]);
        }
    }
}
