﻿using System;
using System.Collections.Generic;
using FourqlParser.Structures;
using FourQLInterpreter.InternalModules;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    [TestClass]
    public class InternalModulesTests : BaseModuleTests
    {
        [TestMethod]
        public void TestMathModule()
        {
            var modules = LoadModules("mathTest.4ql");
            modules.Add("math", new MathModule());
            var evaluator = CreateEvaluator(modules.Values);

            var expression = CreateExpression("int(X) and int(Y) and math.sum(X, Y, 6)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                if (m["X"].Equals(2) && m["Y"].Equals(4))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(4) && m["Y"].Equals(2))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(3) && m["Y"].Equals(3))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else
                    Assert.AreEqual(LogicValue.FALSE, m["logic_value"]);
            }

            expression = CreateExpression("int(X) and int(Y) and math.product(X, Y, 6)", modules, "m");
            result = evaluator.Evaluate(expression, modules["m"]);

            Console.WriteLine(result);
            foreach (var row in result.Rows)
            {
                var m = result.ToFieldMap(row);
                if (m["X"].Equals(1) && m["Y"].Equals(6))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(6) && m["Y"].Equals(1))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(3) && m["Y"].Equals(2))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else if (m["X"].Equals(2) && m["Y"].Equals(3))
                    Assert.AreEqual(LogicValue.TRUE, m["logic_value"]);
                else
                    Assert.AreEqual(LogicValue.FALSE, m["logic_value"]);
            }
        }


        [TestMethod]
        public void TestPlugin()
        {
            PluginModuleWrapper p = new PluginModuleWrapper("../../../4QLInterpreterTest/Resources/Plugins/AlwaysTrue.cs");
            p.Initialize();
            Assert.AreEqual(LogicValue.TRUE, p.GetLiteralValue("r", null));
        }

        [TestMethod]
        public void TestPluginAsModule()
        {
            PluginModuleWrapper p = new PluginModuleWrapper("../../../4QLInterpreterTest/Resources/Plugins/Contains.cs");
            p.Initialize();

            var modules = LoadModules("containsTest.4ql", new List<Module>(){p});
            var evaluator = CreateEvaluator(modules.Values);

            var expression = CreateExpression("str(X) and str(Y) and contains.contains(X, Y)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);
           
        }

        [TestMethod]
        public void TestBrokenModule()
        {
            PluginModuleWrapper p = new PluginModuleWrapper("../../../4QLInterpreterTest/Resources/Plugins/Broken.cs");
            p.Initialize();

            var modules = LoadModules("containsTest.4ql", new List<Module>() { p });
            var evaluator = CreateEvaluator(modules.Values);

            var expression = CreateExpression("str(X) and str(Y) and broken.rel(X, Y)", modules, "m");
            var result = evaluator.Evaluate(expression, modules["m"]);
            Console.WriteLine(result);

        }
    }
}
