﻿using System;
using System.Collections.Generic;
using FourqlParser.Structures;
using FourQLInterpreter.InternalModules;
using FourQLInterpreter.WSM;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace _4QLInterpreterTest
{
    [TestClass]
    public class WSMTest : BaseModuleTests
    {
        [TestMethod]
        public void TestWSMEvaluation()
        {
            var modules = LoadModules("WSMTests/wsmtest1.4ql");
            var hierarchy = new ModuleHierarchy(modules.Values);
            //var moduleGraph = hierarchy.ConstructHierarchy();
            var evaluator = CreateEvaluator(modules.Values);
            
            var wsm = new WsmEvaluator(modules["m"], evaluator, true);
            wsm.ConstructWsm();
            Console.WriteLine(wsm.History);
            Assert.AreEqual(LogicValue.INCONSISTENT,
                evaluator.ModulesData["m"].GetLiteraValue("o", new List<ValueHolder>(){new ValueHolder("x", DomainType.LITERAL)}));
            Assert.AreEqual(LogicValue.INCONSISTENT,
               evaluator.ModulesData["m"].GetLiteraValue("r", new List<ValueHolder>() { new ValueHolder("x", DomainType.LITERAL) }));
            Assert.AreEqual(LogicValue.INCONSISTENT,
                   evaluator.ModulesData["m"].GetLiteraValue("w", new List<ValueHolder>() { new ValueHolder("x", DomainType.LITERAL) }));
        
        }

        [TestMethod]
        public void TestManyItNoIncons()
        {
            var modules = LoadModules("WSMTests/wsm2.4ql");
            var hierarchy = new ModuleHierarchy(modules.Values);
            //var moduleGraph = hierarchy.ConstructHierarchy();
            var evaluator = CreateEvaluator(modules.Values);

           // var expr = CreateExpression("ancestor(X,Z) and ancestor(Z,Y)", modules, "m");
          //  Console.WriteLine(evaluator.Evaluate(expr, modules["m"], new List<string>(){"X", "Y"}, LogicValue.TRUE));
             var wsm = new WsmEvaluator(modules["m"], evaluator, true);
              wsm.ConstructWsm();

             Console.WriteLine(wsm.History.ToString());

        }

        [TestMethod]
        public void Test3()
        {
            var modules = LoadModules("WSMTests/wsm3.4ql");
            var hierarchy = new ModuleHierarchy(modules.Values);
            //var moduleGraph = hierarchy.ConstructHierarchy();
            var evaluator = CreateEvaluator(modules.Values);

            // var expr = CreateExpression("ancestor(X,Z) and ancestor(Z,Y)", modules, "m");
            //  Console.WriteLine(evaluator.Evaluate(expr, modules["m"], new List<string>(){"X", "Y"}, LogicValue.TRUE));
            var wsm = new WsmEvaluator(modules["m0"], evaluator, true);
            wsm.ConstructWsm();

            Console.WriteLine(wsm.History.ToString());

            var wsm1 = new WsmEvaluator(modules["m1"], evaluator, true);
            wsm1.ConstructWsm();

            Console.WriteLine(wsm1.History.ToString());

        }

        [TestMethod]
        public void TestMath()
        {

            var modules = LoadModules("WSMTests/wsm4.4ql", new List<Module>() { new MathModule() });

            var evaluator = CreateEvaluator(modules.Values);

            var wsm = new WsmEvaluator(modules["m"], evaluator, true);
            wsm.ConstructWsm();

            Console.WriteLine(wsm.History.ToString());

        }

        [TestMethod]
        public void TestOrder()
        {

            var modules = LoadModules("WSMTests/wsm5.4ql");

            var evaluator = CreateEvaluator(modules.Values);
            var expr = CreateExpression("ALL X: dom r(X)", modules, "m");
            Console.WriteLine(evaluator.Evaluate(expr, modules["m"], new List<string>()));

           // var wsm = new OptimizedWSMEvaluator(modules["m"], evaluator, true);
            // wsm.ConstructWsm();

            var wsm2 = new WsmEvaluator(modules["m"], evaluator, true);
            wsm2.ConstructWsm();
         
            Console.WriteLine(wsm2.History.ToString());



        }

        [TestMethod]
        public void TestQuantifiers2()
        {

            var modules = LoadModules("quantifierTest.4ql");

            var evaluator = CreateEvaluator(modules.Values);

            var wsm = new OptimizedWSMEvaluator(modules["main"], evaluator, true);
            wsm.ConstructWsm();
            Console.WriteLine(wsm.History.ToString());

            var expr = CreateExpression("EXISTS X: dom (rel(X) and EXISTS Y:dom2 (NOT rel(Y) AND rel(X)))", modules, "main");
            evaluator.Explain = true;
            Console.WriteLine(evaluator.Evaluate(expr, modules["main"], new List<string>()));
       



        }

   

    }
}
