﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using FourqlModeler.Communication;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using FourqlParser.Parsers;
using FourqlParser.StaticAnalysis;
using FourqlParser.Structures;
using FourqlParser.Structures.Print;

namespace FourqlModeler.Calculcations
{
    class CalcExprExecutor
    {
        public Result Execute(string stringExpr, Module execModule, IDictionary<string, Module> modules, params LogicValue[] excludedValues)
        {
            // Prepare an error structure.
            FourqlParser.Parsers.ParserDispatcher.Errors errors = new FourqlParser.Parsers.ParserDispatcher.Errors();

            // Parse the expression and check for errors.
            var exprContext = ParserDispatcher.parseExpr(stringExpr, errors);
            if (errors.ErrorCount > 0)
            {
                // We got some exceptions - combine and send.
                throw new Exception(errors.ToString());
            }

            // Create an expression structure for the expression context.
            var expr = ParserDispatcher.toExpr(exprContext);
            MainWindowViewModel.Instance.ConsoleViewModel.SendDebugCommand(new ExprPrinter().ConvertExpr(expr).ToString());

            try
            {
                // Run the checks on the expression structure.
                ParserDispatcher.checkExpr(expr, modules.Values, execModule);
            }
            catch (Exception ex)
            {
                // Checks failed - throw a combined message.
                throw new Exception(ExceptionUtils.GetMessagesWithInners(ex));
            }

            // Execute the expression
            var result = ApplicationClient.Instance.Query((execModule == null? "" : execModule.Name +".") + "("+ new ExprPrinter().ConvertExpr(expr, false).ToString() + ")");
            

            // filter results
            var filteredResult = new Result()
            {
                Fields = result.Fields,
                Rows = new ObservableCollection<Row>(result.Rows.Where(r => !excludedValues.Contains(r.Value)))
            };
            return filteredResult;
        }

        public Result Execute(Rule rule, Module execModule, IDictionary<string, Module> modules, params LogicValue[] excludedValues)
        {
            MainWindowViewModel.Instance.ConsoleViewModel.SendDebugCommand(new ProgramPrinter().append(rule).Done());

            try
            {
                // Run the checks on the rule structure.
                ParserDispatcher.checkRule(rule, modules.Values, execModule);
            }
            catch (Exception ex)
            {
                // Checks failed - throw a combined message.
                throw new Exception(ExceptionUtils.GetMessagesWithInners(ex));
            }

            // execute rule body
            var result = Execute(new ExprPrinter().ConvertExpr(rule.Body).ToString(), execModule, modules, excludedValues);

            return result;
        }

        public void FilterResult(Result result, string varName, IEnumerable<object> varValues)
        {
           
            result.Rows = new ObservableCollection<Row>(
                    from row in result.Rows
                    where result.ToFieldMap(row).ContainsKey(varName) && varValues.Contains(result.ToFieldMap(row)[varName])
                    select row
                    );
        }

    }

}
