﻿
using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;
using System.Collections.ObjectModel;
using FourQLInterpreter.Reasoning.Explanation;

namespace FourqlModeler.Calculcations
{
    public class Result
    {
        public List<Field> Fields { get; set; }
        public ObservableCollection<Row> Rows { get; set; }

        public Result()
        {
            Fields = new List<Field>();
            Rows = new ObservableCollection<Row>();
        }

        public IDictionary<string, object> ToFieldMap(Row row)
        {
            var fieldMap = new Dictionary<string, object>();
            var fieldIter = Fields.GetEnumerator();
            var valueIter = row.Values.GetEnumerator();

            while (fieldIter.MoveNext() && valueIter.MoveNext())
            {
                fieldMap.Add(fieldIter.Current.Name, valueIter.Current);
            }

            return fieldMap;
        }

        public Row AddDummyRow()
        {
            var row = new Row() { IsDummyRow = true, Value = LogicValue.UNKNOWN};
            foreach (var field in Fields)
            {
                row.Values.Add("EMPTY DOMAIN FOR AT LEAST ONE VARIABLE");
            }
            Rows.Add(row);
            return row;
        }

        public override string ToString()
        {
            var builder = new StringBuilder("Fields:");
            foreach (var field in Fields)
            {
                builder.Append(String.Format(" {0}:{1}", field.Name, DomainTypeUtils.getName(field.Type)));
            }

            builder.Append(String.Format("\nRows ({0}):", Rows.Count));
            foreach (var row in Rows)
            {
                builder.Append("\n\tValues=[");
                foreach (var value in row.Values)
                {
                    builder.Append(String.Format(" '{0}'", value));
                }
                builder.Append(String.Format(" ] Logical Value={0}", row.Value));
            }

            return builder.ToString();
        }
    }

    public class Field
    {
        public string Name { get; set; }
        [XmlIgnoreAttribute]
        public DomainType Type { get; set; }

        public override string ToString()
        {
            return Name + ":" + DomainTypeUtils.getName(Type);
        }
    }

    public class Row
    {
        [XmlIgnoreAttribute]
        public bool IsDummyRow { get; set; }
        public List<Object> Values { get; set; }
        public LogicValue Value { get; set; }
        [XmlIgnoreAttribute]
        public Explanation Explanation { get; set; }

        public Row()
        {
            Values = new List<Object>();
        }

        public override string ToString()
        {
            var builder = new StringBuilder("Values=[");
            foreach (var value in Values)
            {
                builder.Append(String.Format(" '{0}'", value));
            }
            builder.Append(String.Format(" ] Logical Value={0}", Value));
            return builder.ToString();
        }
    }

    /*public class Explanation
    {
        public ICollection<LogicValue> LiteralValues { get; set; }
        public LogicValue ExprValue { get; set; }

        public Explanation()
        {
            LiteralValues = new List<LogicValue>();
            ExprValue = LogicValue.UNKNOWN;
        }
    }
     * */
}
