﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using FourqlModeler.Calculcations;
using FourqlModeler.Communication;
using FourqlModeler.Communication.Import;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using FourqlModeler.Views;
using FourqlParser.Parsers;
using Parago.Windows;

namespace FourqlModeler.Commands.Console
{
    class ExecuteExpressionCommand : ICommand
    {
        #region ICommand Members

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            var inputBox = parameter as TextBox;
            if (inputBox != null)
            {
                try
                {
                    if (ExecuteExpr(inputBox.Text))
                    {
                        inputBox.Text = "";
                    }
                }
                catch (Exception ex)
                {
                    inputBox.Text = "";
                    throw ex;
                }
            }
        }
        
        #endregion

        public static bool ExecuteExpr(string strExpr)
        {
            if (string.IsNullOrWhiteSpace(strExpr))
            {
                MessageBox.Show("Expressions cannot be empty.", "Empty expression", MessageBoxButton.OK, MessageBoxImage.Warning);
                return false;
            }
            var errors = new ParserDispatcher.Errors();
            var exprContext = ParserDispatcher.parseExpr(strExpr, errors);
            if (errors.ErrorCount > 0)
            {
                MessageBox.Show(errors.ToString(), "Parser error", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            string errorMessage = null;
            string errorHeader = null;
            ProgressDialog.Execute(App.Current.MainWindow, "Executing the expression...", () =>
            {
                try
                {
                    ProgressDialog.Current.ReportWithCancellationCheck("Checking the expression...");
                    var expr = ParserDispatcher.toExpr(exprContext);
                    ParserDispatcher.checkExpr(expr, ModuleRepository.Instance.Modules.Values, null);
                    try
                    {
                        MainWindowViewModel.Instance.ConsoleViewModel.SendCommand("Executing: " + strExpr);
                        App.Current.Dispatcher.Invoke(() =>
                        {
                            ProgressDialog.Current.ReportWithCancellationCheck("Executing the expression...");
                        });
                        var result = new CalcExprExecutor().Execute(strExpr, null, ModuleRepository.Instance.Modules);
                        App.Current.Dispatcher.Invoke(() =>
                        {
                            MainWindowViewModel.Instance.ConsoleViewModel.SendCommand("Final result is=\n" + result.ToString());
                        });
                        var tab = strExpr + "#" + DateTime.Now.ToLongTimeString();
                        ProgressDialog.Current.ReportWithCancellationCheck("Creating a result tab for the expression...");
                        App.Current.Dispatcher.Invoke(() =>
                        {
                            MainWindowViewModel.Instance.MainWindowView.TabController.createTab(tab, new ResultView(result, expr, null));
                            MainWindowViewModel.Instance.MainWindowView.TabController.selectTab(tab);
                        });
                    }
                    catch (Exception ex)
                    {
                        errorHeader = "Rule error";
                        errorMessage = ExceptionUtils.GetMessagesWithInners(ex);
                    }
                }
                catch (Exception ex)
                {
                    errorHeader = "Static analysis error";
                    errorMessage = ExceptionUtils.GetMessagesWithInners(ex);
                }
            }, ProgressDialogSettings.WithSubLabelAndCancel);
            if (errorMessage != null)
            {
                MessageBox.Show(errorMessage, errorHeader, MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            return true;
        }
    }
}
