﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using FourqlModeler.Views.ValueConverters;

namespace FourqlModeler.Commands.ErrorGrouping
{
    class ApplyGroupingByIdCommand : ICommand
    {
        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            var grid = parameter as DataGrid;
            if (grid != null)
            {
                var collectionView = CollectionViewSource.GetDefaultView(grid.ItemsSource);
                if (collectionView != null)
                {
                    collectionView.GroupDescriptions.Clear();
                    collectionView.GroupDescriptions.Add(new PropertyGroupDescription("GroupId"));
                }

                if (grid.Columns.Where(col => col.Header.Equals("Description")).Count() == 0)
                {
                    grid.Columns.Add(new DataGridTextColumn() { Header = "Description", Width = new DataGridLength(1.0, DataGridLengthUnitType.Star), Binding = new Binding("Description") });
                }
            }
        }

        #endregion
    }
}
