﻿using FourqlModeler.Dialogs;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using FourqlParser.Parsers;
using Parago.Windows;
using System.Windows.Threading;
using System.IO;

namespace FourqlModeler.Commands
{
    public class LoadModulesCommand : ICommand
    {
        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }

        public LoadModulesCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            string errorMessage = null;
            var parsedModules = new Dictionary<int, ModuleViewModel>();
            var result = ProgressDialog.Execute(Application.Current.MainWindow, "Loading modules", () =>
            {
                try
                {
                    ProgressDialog.Current.Report("Waiting for files to load...");
                    var files = FileDialogs.OpenFilesWithDialog();
                    if (files.Count == 0)
                    {
                        return;
                    }

                    int i = 0;
                    foreach (var file in files)
                    {
                        var fileName = Path.GetFileName(file.Key);
                        var fileDir = Path.GetDirectoryName(file.Key);
                        if (!fileDir.EndsWith("\\") && !fileDir.EndsWith("/"))
                        {
                            fileDir = fileDir + "\\";
                        }
                        ModuleViewModel moduleViewModel = null;
                        foreach (var existingModel in ModuleStorageViewModel.Modules)
                        {
                            if (existingModel.FileName.Equals(fileName) && existingModel.FileDir.Equals(fileDir))
                            {
                                moduleViewModel = existingModel;
                                break;
                            }
                        }

                        ++i;
                        if (moduleViewModel == null)
                        {
                            moduleViewModel = new ModuleViewModel() { FileName = fileName, FileDir = fileDir, Content = file.Value, SavedContent = file.Value, IsDirty = false };
                            parsedModules.Add(i, moduleViewModel);
                        }
                        else
                        {
                            parsedModules.Add(-i, moduleViewModel);
                        }
                    }

                    Application.Current.Dispatcher.Invoke(() =>
                    {
                        foreach (var loadedModule in parsedModules.Where(entry => entry.Key > 0))
                        {
                            ModuleStorageViewModel.Modules.Add(loadedModule.Value);
                        }
                    });
                }
                catch (Exception ex)
                {
                    errorMessage = ex.Message;
                }
            }, ProgressDialogSettings.WithSubLabel);

            if (errorMessage != null)
            {
                MessageBox.Show("Failed to load the 4QL files: " + errorMessage, "Load failed");
            }
            else if (parsedModules.Count > 0)
            {
                ModuleStorageViewModel.BuildModulesCommand.Execute(parsedModules.Where(entry => entry.Key > 0).Select(entry => entry.Value).ToList());
                result = ProgressDialog.Execute(Application.Current.Windows[0], "Opening loaded modules", () =>
                    {
                        ProgressDialog.Current.Report("Opening tabs if requested.", parsedModules.Count);
                        Application.Current.Dispatcher.Invoke(() =>
                        {
                            foreach (var entry in parsedModules)
                            {
                                var moduleViewModel = entry.Value;
                                MainWindowViewModel.Instance.MainWindowView.OpenModuleEvent(moduleViewModel);
                            }
                        });
                    });

                if (result.OperationFailed)
                {
                    MessageBox.Show("Failed to open the 4QL files: " + (result.Error != null ? result.Error.Message : result.Result), "Open failed");
                }
            }
        }

        #endregion
    }
}
