﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using FourqlModeler.ViewModels;

namespace FourqlModeler.Commands
{
    public abstract class ModuleViewModelCommand : ICommand
    {
        #region ICommand Members

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            var moduleViewModels = parameter as ICollection<ModuleViewModel>;
            if (moduleViewModels != null)
            {
                foreach (var moduleViewModel in moduleViewModels)
                {
                    ExecuteOn(moduleViewModel);
                }
            }
            else
            {
                var items = parameter as ICollection;
                if (items != null)
                {
                    foreach (var item in items)
                    {
                        var element = item as FrameworkElement;
                        if (element != null)
                        {
                            var moduleViewModel = element.DataContext as ModuleViewModel;
                            if (moduleViewModel != null)
                            {
                                ExecuteOn(moduleViewModel);
                            }
                        }
                    }
                }
                else
                {
                    var moduleViewModel = parameter as ModuleViewModel;
                    if (moduleViewModel != null)
                    {
                        ExecuteOn(moduleViewModel);
                    }
                }
            }
        }

        protected abstract void ExecuteOn(ModuleViewModel moduleViewModel);

        #endregion
    }
}
