﻿using FourqlModeler.Dialogs;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace FourqlModeler.Commands
{
    public class NewModuleCommand : ICommand
    {
        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }

        public NewModuleCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            try
            {
                var filePath = FileDialogs.GetSaveFileWithDialog();
                if (String.IsNullOrEmpty(filePath))
                {
                    return;
                }

                foreach (var existingModel in ModuleStorageViewModel.Modules)
                {
                    var fullModulePath = existingModel.FileDir + existingModel.FileName;
                    if (fullModulePath.Equals(filePath))
                    {
                        // We tried to create a module in place of our already-remembered module.
                        MessageBox.Show(String.Format("Cannot create a new module file in {0} as the file is loaded into the program memory. Unload it first and then re-try or use another file instead.", filePath), "Cannot create a new module file", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                        return;
                    }
                }

                var content = parameter != null ? parameter.ToString() : "";
                var fileName = Path.GetFileName(filePath);
                var fileDir = Path.GetDirectoryName(filePath);
                if (!fileDir.EndsWith("\\") && !fileDir.EndsWith("/"))
                {
                    fileDir = fileDir + "\\";
                }

                MainWindowViewModel.Instance.ConsoleViewModel.SendDebugCommand(fileName + " " + fileDir);

                FileDialogs.SaveFile(filePath, content);

                var moduleViewModel = new ModuleViewModel() { FileName = fileName, FileDir = fileDir, SavedContent = content, Content = content, IsDirty = false };
                ModuleStorageViewModel.Modules.Add(moduleViewModel);

                if (parameter != null)
                {
                    ModuleStorageViewModel.BuildModulesCommand.Execute(new List<ModuleViewModel>() { moduleViewModel });
                }

                MainWindowViewModel.Instance.MainWindowView.OpenModuleEvent(moduleViewModel);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ExceptionUtils.GetMessagesWithInners(ex), "Create new error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        #endregion
    }
}
