﻿using FourqlModeler.Dialogs;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace FourqlModeler.Commands
{
    public class NewSampleModuleCommand : ICommand
    {
        private static readonly string SampleModule =
            "/**\n" +
            " * This is a sample module to fill.\n" +
            " */\n" +
            "module main {\n" +
            "\t// This section contains domains from which variables take values.\n" +
            "\t// Defined domains are always of primitive types, but you can use\n" +
            "\t// primitive types in relation definitions as well. Domains get\n" +
            "\t// their values from different (implied by rules or base (module)) facts\n" +
            "\t// and there is no other relation between domain values than facts (ex. having\n" +
            "\t// a value in a defined domain of some base domain type does not mean\n" +
            "\t// that the base domain also contains the value).\n" +
            "\t// Defined types start with a small letter.\n" +
            "\t// Primitive types: literal, logic, string, integer, real, date, date_time.\n" +
            "\t// Defined types share the same value patterns as their primitive relatives.\n" +
            "\t// Sample values:\n" +
            "\t//  * literal:   a, b, c, _abc (starts with a small letter or a '_' and must contains one alphanumeric sign),\n" +
            "\t//  * logic:     true, false, unknown, incons/inconsistent (those exhaust the case),\n" +
            "\t//  * string:    \"a\", \"b\", \"c\", \"abc\",\n" +
            "\t//  * integer:   -1, 0, 1, 7,\n" +
            "\t//  * real:      -1., -1.0, 0.0, 1.0, 1.,\n" +
            "\t//  * date:      01-09-1939, (two digits for day, two digits for month, four for year),\n" +
            "\t//  * date_time: 01-09-1939 01:01, (as date with two digits for hour and two for minute).\n" +
            "\tdomains {\n" +
            "\t\t// sampleLiteralDomain : literal;\n" +
            "\t}\n" +
            "\t// Relations the concepts of 4QL and stand for a base for facts. Those come from base module facts\n" +
            "\t// and rules of relations. Relations are defined by stating their names and\n" +
            "\t// parameter (domain) types.\n" +
            "\trelations {\n" +
            "\t\t// sampleRelation(literal, sampleLiteralDomain);\n\n" +
            "\t// Rules stand for a model of calculations of facts. They use a sophisticated algorithm to\n" +
            "\t// imply proper fact values. A rule implies facts when:\n" +
            "\t//   a) its body states that for grounded values (those from rule's head) it gets\n" +
            "\t//      INCONSISTENT - imply INCONSISTENT for the grounded values from the head,\n" +
            "\t//   b) its body states that for grounded values (those from rule's head) it gets\n" +
            "\t//      TRUE - imply TRUE/FALSE when its head is positive/negative (respectively),\n" + 
            "\t//   c) otherwise - imply nothing.\n" +
            "\t// Facts that are negative and positive are inconsistent.\n" +
            "\t// Rule heads can have constants and variables. Rule body is an expression consisting of alternatives, \n" +
            "\t// conjunctions, negations, quantifiers etc. All variables from rule head must also appear in the body. \n" +
            "\t// Variables always start with a capital letter. Modules can access other modules in their rules,\n" +
            "\t// but there can be no circular dependencies on between modules - meaning module 'm1' can have a\n" +
            "\t// rule that depends on module 'm2', but in that case module 'm2' cannot depend on any module that\n" +
            "\t// depends on module 'm1' (the relation is transitive), but also 'm2' cannot depend on 'm1' as well.\n" +
            "\t// Variable domains are intersections of all domains that represent variables.\n"  +
            "\t// Alternative is MIN and conjunction is MAX on the value order FALSE < UNKNOWN < INCONSISTENT < TRUE.\n" +
            "\t// Negation is ID for UNKNOWN and INCONSISTENT and TRUE -> FALSE, FALSE -> TRUE for the two.\n" +
            "\t// IN casts fact values to TRUE/FALSE according to what the IN construction contains in desired values.\n" +
            "\t// ALL X:dom r(X) is equal to AND(r(X1), r(X2), r(X3), ...) where X1, X2, X3, ... are all the values in domain 'dom'. If 'dom' is empty, value of ALL is TRUE \n" +
            "\t// EXISTS X:dom r(X) is equal to OR(r(X1), r(X2), r(X3), ...) where X1, X2, X3, ... are all the values in domain 'dom'. If 'dom' is empty, value of EXISTS is FALSE \n" +
            "\t}\n" +
            "\trules {\n" +
            "\t\t//sampleRelation(a, X) WHEN NOT sampleRelation(X, X) AND inner.rel(X, X) in { true } OR sampleRelation(X, X) AND NOT inner.rel(X, X) in { true } ;\n" +
            "\t}\n" +
            "\t// Facts stand for base knowledge of modules about themselves.\n" +
            "\t// Facts are defined according to local relations and can only\n" +
            "\t// use constants as parameters.\n" +
            "\tfacts {\n" +
            "\t\t// The below sample says that sampleRelation(a, abc) = INCONS as\n" +
            "\t\t// we have one positive and one negative fact for the relation and\n" +
            "\t\t// the set of parameters.\n" +
            "\t\t//sampleRelation(a, abc);\n" +
            "\t\t//NOT sampleRelation(a, abc);\n" +
            "\t}\n" +
            "}";

        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }

        public NewSampleModuleCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            ModuleStorageViewModel.NewModuleCommand.Execute(SampleModule);
        }

        #endregion
    }
}
