﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using FourqlModeler.ViewModels;
using Parago.Windows;

namespace FourqlModeler.Commands
{
    public class SaveAllModulesCommand : ICommand
    {
        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }
        private ICommand SaveModuleCommand { get; set; }

        public SaveAllModulesCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
            SaveModuleCommand = new SaveModuleCommand(moduleStorageViewModel, false);
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            if (ModuleStorageViewModel.Modules.Count == 0)
            {
                return;
            }

            string errorMessage = null;
            var changedModules = new List<ModuleViewModel>();
            var result = ProgressDialog.Execute(Application.Current.MainWindow, "Saving all modules", () =>
            {
                try
                {
                    int i = 0;
                    foreach (var moduleViewModel in ModuleStorageViewModel.Modules)
                    {
                        ProgressDialog.Current.Report("Saving {0}/{1} file ...\nFile name: {2}\nFile dir: {3}", ++i, ModuleStorageViewModel.Modules.Count, moduleViewModel.FileName, moduleViewModel.FileDir);
                        if (moduleViewModel.IsDirty)
                        {
                            SaveModuleCommand.Execute(moduleViewModel);
                            changedModules.Add(moduleViewModel);
                        }
                    }
                }
                catch (Exception ex)
                {
                    errorMessage = ex.Message;
                }
            }, ProgressDialogSettings.WithSubLabel);

            if (errorMessage != null)
            {
                MessageBox.Show("Failed to save the 4QL files: " + errorMessage, "Save failed");
            }
            else
            {
                ModuleStorageViewModel.BuildModulesCommand.Execute(changedModules);
            }
        }

        #endregion
    }
}
