﻿using FourqlModeler.Dialogs;
using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Media;
using Xceed.Wpf.AvalonDock.Layout;

namespace FourqlModeler.Commands
{
    public class SaveFocusedModuleCommand : ICommand
    {
        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }

        public SaveFocusedModuleCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            var focusedTab = MainWindowViewModel.Instance.MainWindowView.GetFocusedTab();
            if (focusedTab != null)
            {
                // FIXME: Disabled - we let save any "last opened/focused tab". Somewhy the graph view is not even displayed in the logical/visual trees...
                //checkTabFocus(focusedTab);
                
                var filePath = focusedTab.Title.Substring(0, focusedTab.Title.IndexOf("#"));
                foreach (var entry in ModuleStorageViewModel.Modules)
                {
                    if (filePath.Equals(entry.FileDir + entry.FileName))
                    {
                        // FIXME: MODULES, GRAPHS, MULTIPLE AFTER-HASH OPTIONS CAN BE USED.
                        ModuleStorageViewModel.SaveModuleCommand.Execute(entry);
                        MessageBox.Show("Saved the tab: " + focusedTab.Title + " as a module file!\n\nFile: " + entry.FileName + "\nDirectory: " + entry.FileDir, "Saved the module file");
                        break;
                    }
                }
            }
        }

        private void checkTabFocus(LayoutDocument focusedTab)
        {
            var tabView = focusedTab.Content as UserControl;
            if (tabView == null)
            {
                MessageBox.Show("Not a view inside.");
                return;
            }

            var focusScope = FocusManager.GetFocusScope(tabView);
            if (focusScope == null)
            {
                MessageBox.Show("No focus scope.");
                return;
            }

            Debug.Assert(false, "Focus scope: " + focusScope);
            var focusedElement = FocusManager.GetFocusedElement(focusScope);
            if (focusedElement == null)
            {
                Debug.Assert(false, "No focused element.");
                return;
            }

            var current = focusedElement as DependencyObject;
            if (current == null)
            {
                MessageBox.Show("Focused element not dependency object.");
            }

            System.Console.WriteLine("Search for: " + tabView + "\n  or: " + focusedTab + " " + "\n  starting from: " + current + "\n  and more: " + focusedTab.Parent + "\n  and more: " + MainWindowViewModel.Instance.MainWindowView.TabController.TabPanel + "\n  and more: " + MainWindowViewModel.Instance.MainWindowView.TabController.TabPanelParent);
            var parent = current;
            while (parent != tabView && parent != focusedTab && parent != MainWindowViewModel.Instance.MainWindowView.TabController.TabPanel && parent != MainWindowViewModel.Instance.MainWindowView.TabController.TabPanelParent)
            {
                var next = LogicalTreeHelper.GetParent(parent);
                if (next == null)
                {
                    next = VisualTreeHelper.GetParent(parent);
                }

                System.Console.WriteLine("Checking parent of: " + parent + "\n  moving to: " + next);
                if (next == null)
                {
                    MessageBox.Show("Reached the end of the tree and did not find the view.");
                    return;
                }

                parent = next;
            }

            MessageBox.Show("Focused: " + focusedElement);
        }

        #endregion
    }
}
