﻿using System;
using System.Windows;
using System.Windows.Input;
using FourqlModeler.Controls;
using FourqlModeler.ViewModels;
using FourqlModeler.Views;
using ICSharpCode.AvalonEdit.Editing;

namespace FourqlModeler.Commands
{
    public class ShowErrorSpotCommand : ICommand
    {
        public ModuleStorageViewModel ModuleStorageViewModel { get; private set; }

        public ShowErrorSpotCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            Application.Current.Dispatcher.Invoke(() =>
            {
                // Mark the currently selected error now.
                var error = ModuleStorageViewModel.SelectedError;
                if (error != null)
                {
                    ModuleStorageViewModel.OpenModuleCommand.Execute(error.ModelSource);
                    var tab = MainWindowViewModel.Instance.MainWindowView.TabController.GetTab(error.ModelSource.FileDir + error.ModelSource.FileName + "#def");
                    if (tab != null)
                    {
                        var view = tab.Content as ModuleEditorView;
                        if (view != null)
                        {
                            view.EditorBox.ScrollTo(error.StartLine, error.StartColumn + 1);

                            var startOffset = OffsetOf(error.StartLine, error.StartColumn, view.EditorBox.TextArea);
                            var endOffset = error.EndLine <= 0 || error.EndColumn < 0
                                ? OffsetOf(error.StartLine, int.MaxValue, view.EditorBox.TextArea)
                                : OffsetOf(error.EndLine, error.EndColumn + 1, view.EditorBox.TextArea);
                            view.EditorBox.Select(startOffset, endOffset - startOffset);
                        }
                    }
                }
            });
        }

        #endregion

        public int OffsetOf(int line, int column, TextArea textArea)
        {
            if (line <= 0 || line > textArea.Document.LineCount)
            {
                return textArea.Document.GetLineByNumber(textArea.Document.LineCount).EndOffset;
            }
            var lineStruct = textArea.Document.GetLineByNumber(line);
            if (column >= 0 && column < lineStruct.Length)
            {
                return lineStruct.Offset + column;
            }
            return lineStruct.EndOffset;
        }
    }
}
