﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using FourqlModeler.Calculcations;
using FourqlModeler.Communication.Import;
using FourqlModeler.Utility;
using FourqlModeler.ViewModels;
using Parago.Windows;

namespace FourqlModeler.Commands.Tools
{
    class ShowModuleGraphCommand : ICommand
    {
        private ModuleStorageViewModel ModuleStorageViewModel { get; set; }

        public ShowModuleGraphCommand(ModuleStorageViewModel moduleStorageViewModel)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
        }

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public void Execute(object parameter)
        {
            var result = ProgressDialog.Execute(Application.Current.MainWindow, "Building a module graph...", () =>
            {
                // Add the initial message.
                ProgressDialog.Current.Report("Building a reasoning order...");
                // Load the parsed structures to our module memory.
                var builder = new ReasoningOrderBuilder();
                Application.Current.Dispatcher.Invoke(() =>
                {
                    foreach (var module in ModuleStorageViewModel.Modules)
                    {
                        builder.AddModel(module);
                    }
                });
                var reasoningOrder = builder.Done();
                ProgressDialog.Current.Report("Displaying the reasoning order...");
                // Display the reasoning order.
                Application.Current.Dispatcher.Invoke(() =>
                {
                    // Display the modules as a graph.
                    MainWindowViewModel.Instance.MainWindowView.CreateHierarchyGraphFor(reasoningOrder, parameter != null ? bool.Parse(parameter.ToString()) : false);
                });
            }, ProgressDialogSettings.WithSubLabel);

            if (result.OperationFailed)
            {
                MessageBox.Show("Failed to create a module graph: " + (result.Error != null ? result.Error.Message : result.Result), "Module graph failed to create");
            }
        }

        #endregion
    }
}
