﻿using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Windows.Threading;
using System.Windows;
using Xceed.Wpf.AvalonDock.Layout;
using System.Collections.ObjectModel;

namespace FourqlModeler.Commands
{
    public class UnloadModuleCommand : ModuleViewModelCommand
    {
        public ModuleStorageViewModel ModuleStorageViewModel { get; set; }
        public bool ReparseAfterExecution { get; set; }
        public ICommand SaveModuleCommand { get; set; }

        public UnloadModuleCommand(ModuleStorageViewModel moduleStorageViewModel, bool reparseAfterExecution = true)
        {
            ModuleStorageViewModel = moduleStorageViewModel;
            ReparseAfterExecution = reparseAfterExecution;
            SaveModuleCommand = new SaveModuleCommand(ModuleStorageViewModel, false);
        }

        #region ModuleViewModelCommand Members

        protected override void ExecuteOn(ModuleViewModel moduleViewModel)
        {
            if (moduleViewModel.IsDirty)
            {
                var result = MessageBox.Show(String.Format("Some changes in {0}{1} remain unsaved. Do you want to save those before unloading the file, unload it anyway or cancel the operation?", moduleViewModel.FileDir, moduleViewModel.FileName), "Changes detected", MessageBoxButton.YesNoCancel);
                switch (result)
                {
                    case MessageBoxResult.Yes:
                        SaveModuleCommand.Execute(moduleViewModel);
                        break;
                    case MessageBoxResult.No:
                        break;
                    default:
                        return;
                }
            }

            ModuleStorageViewModel.Modules.Remove(moduleViewModel);
            var filePath = moduleViewModel.FileDir + moduleViewModel.FileName + "#";
            var tabs = MainWindowViewModel.Instance.MainWindowView.TabController.OpenTabs.Where(entry => entry.Key.StartsWith(filePath)).Select(entry => entry.Value).ToArray<LayoutDocument>();
            foreach (var tab in tabs)
            {
                Application.Current.Dispatcher.Invoke(() => tab.Close());
            }

            if (ReparseAfterExecution)
            {
                ModuleStorageViewModel.BuildModulesCommand.Execute(new List<ModuleViewModel>());
            }
        }

        #endregion
    }
}
