﻿
using FourqlModeler.ViewModels;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;
using FourqlModeler.Calculcations;
using FourqlModeler.Utility;
using FourqlParser.Structures.Print;
using FourQLInterpreter.Application;
using Mono.CSharp.Linq;

namespace FourqlModeler.Communication
{
    public class ApplicationClient
    {
        private static readonly int MetaSize = 8;
        private static readonly int PacketSize = 1024;
        private static readonly int PartSize = PacketSize - MetaSize;

        private static ApplicationClient _Instance = new ApplicationClient();
        public static ApplicationClient Instance { get { return _Instance; } }

        private string HostName { get; set; }
        private int HostPort { get; set; }
        private Socket ClientSocket { get; set; }

        public Interpreter InterpreterInstance { get; private set; }

        private ApplicationClient()
        {
            InterpreterInstance = new Interpreter();
        }


        public void connect()
        {
            InterpreterInstance.Initialize();
        }


        public void shutDown()
        {

        }


        public Result Query(string query, Boolean explain = true)
        {
            try
            {
                InterpreterInstance.ExpressionEvaluator.Explain = explain;
                var result = InterpreterInstance.Query(query);
                return Convert(result);
            }
            catch (Exception error)
            {
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(String.Format("Error of Query '{0}' : {1}", query, ExceptionUtils.GetMessagesWithInners(error)));
                throw error;
            }
        }


        private Result Convert(FourQLInterpreter.Reasoning.Result res)
        {
            var f = from c in res.ColumnHeaders
                select new Field() {Name = c.Name, Type = c.Type};
            var r = from row in res.Rows
                select new Row() {Value = row.Value, Values = (from v in row.Values select v.Value).ToList(), Explanation = row.RowExplanation};
            var result = new Result() {Fields = new List<Field>(f), Rows = new ObservableCollection<Row>(r)};
            return result;
        }

        public void ClearModules()
        {
            try
            {
                InterpreterInstance.Clear();
            }
            catch (Exception error)
            {
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(String.Format("Error {0} of Clear: {1}", error.GetType(), ExceptionUtils.GetMessagesWithInners(error)));
            }
        }

        public void LoadModules(string path)
        {
            try
            {
                InterpreterInstance.LoadModules(path);
            }
            catch (Exception error)
            {
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(String.Format("Error {0} of LoadModules: {1}, {2}", error.GetType(), ExceptionUtils.GetMessagesWithInners(error), error.StackTrace));
                throw error;
            }
        }

        public string PrintModule(string module)
        {
            try
            {
                var m = InterpreterInstance.Modules[module];
                var print = new ProgramPrinter().append(m, "  ").Done();
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(String.Format("Success of Print - Print message gave this module definition from inter4ql:\n{0}", print));
                return print;
            }
            catch (Exception error)
            {
                var errorMessage = String.Format("Could not fetch internal/external module definition from inter4ql: {0}", ExceptionUtils.GetMessagesWithInners(error));
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(errorMessage);
                throw  new Exception(errorMessage);
            }
            
        }

        public IList<string> GetModules()
        {
           return (from module in InterpreterInstance.Modules.Values 
                  where !module.Name.StartsWith("_")
                  select module.Name).ToList();
        }
    }
}
