﻿using FourqlModeler.Calculcations;
using FourqlModeler.ViewModels;
using FourqlModeler.Utility;
using FourqlParser.Parsers;
using FourqlParser.Structures;
using FourqlParser.Structures.Print;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlModeler.Communication.Import
{
    class ModuleReader
    {
        public Program parseAndPush(string fileName, IDictionary<string, Module> modules)
        {
            var errors = new FourqlParser.Parsers.ParserDispatcher.Errors();
            var programContext = ParserDispatcher.parseProgram(ParserDispatcher.AsFile(fileName), errors, null);
            if (errors.ErrorCount > 0)
            {
                // We got some exceptions - combine and send.
                throw new Exception(errors.ToString());
            }

            var program = ParserDispatcher.toProgram(programContext);
            MainWindowViewModel.Instance.ConsoleViewModel.SendDebugCommand(new ProgramPrinter().append(program).Done());

            try
            {
                // Run the checks on the program structure.
                ParserDispatcher.checkProgram(program);
            }
            catch (Exception ex)
            {
                // Checks failed - throw a combined message.
                throw new Exception(ExceptionUtils.GetMessagesWithInners(ex));
            }
            
            foreach (var module in program.Modules) {
                //var domainModule = new DomainModuleBuilderNoOpt().CreateDomainModule(module);

                modules[module.Name] = module;
                //modules[domainModule.Name] = domainModule;
            }
            
            return program;
        }

        public void createCombinedModuleFile(string fileName, IDictionary<string, Module> modules)
        {
            // Delete an existing file if necessary as we want to write a clean copy of what is in the repository.
            if (File.Exists(fileName)) {
                File.Delete(fileName);
            }

            // Create an empty file and wrap its stream to start writing to it.
            var fileStream = File.Create(fileName);
            var writer = new StreamWriter(fileStream);

            // Append all modules to a string builder.
            var builder = new StringBuilder();
            foreach (var module in modules.Values) {
                builder.AppendLine(new ProgramPrinter().append(module).Done());
            }

            // Write the module repository to the prepared empty file and flush the stream.
            writer.Write(builder.ToString());
            writer.Flush();

            // Close the writer.
            writer.Close();
        }
    }
}
