﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlModeler.Dialogs
{
    class FileDialogs
    {
        public static IDictionary<string, string> OpenFilesWithDialog()
        {
            var dialog = new OpenFileDialog() { Title = "Select 4QL module files to add to 4qlModeler", DefaultExt = ".4ql", Filter = "4QL Files (.4ql)|*.4ql", CheckFileExists = true, CheckPathExists = true, Multiselect = true };
            var result = dialog.ShowDialog();
            var readFiles = new Dictionary<string, string>();
            if (result.HasValue && result == true)
            {
                var fileNames = dialog.FileNames;
                var fileStreams = dialog.OpenFiles();
                for (int i = 0; i < dialog.FileNames.Length; ++i)
                {
                    using (var fileStream = fileStreams[i])
                    {
                        using (var reader = new StreamReader(fileStream))
                        {
                            readFiles.Add(fileNames[i], reader.ReadToEnd());
                            reader.Close();
                        }
                        fileStream.Close();
                    }
                }
            }
            return readFiles;
        }

        public static ICollection<string> GetOpenFilesWithDialog()
        {
            var dialog = new OpenFileDialog() { Title = "Select 4QL module files to add to 4qlModeler", DefaultExt = ".4ql", Filter = "4QL Files (.4ql)|*.4ql", CheckFileExists = true, CheckPathExists = true, Multiselect = true };
            var result = dialog.ShowDialog();
            var files = new List<string>();
            if (result.HasValue && result == true)
            {
                foreach (string fileName in dialog.FileNames)
                {
                    files.Add(fileName);
                }
            }
            return files;
        }

        public static ICollection<string> OpenFiles(ICollection<string> fileNames)
        {
            var fileContents = new List<string>();
            foreach (var fileName in fileNames)
            {
                using (var fileStream = new FileStream(fileName, FileMode.Open))
                {
                    using (var reader = new StreamReader(fileStream))
                    {
                        fileContents.Add(reader.ReadToEnd());
                        reader.Close();
                    }
                    fileStream.Close();
                }
            }
            return fileContents;
        }

        public static void SaveFileWithDialog(string content)
        {
            var dialog = new SaveFileDialog() { Title = "Select a file to write the 4QL module to", DefaultExt = ".4ql", Filter = "4QL Files (.4ql)|*.4ql", AddExtension = true, OverwritePrompt = true, CheckPathExists = true };
            var result = dialog.ShowDialog();

            if (result.HasValue && result == true)
            {
                using (var fileStream = dialog.OpenFile())
                {
                    using (var writer = new StreamWriter(fileStream))
                    {
                        writer.Write(content);
                        writer.Close();
                    }
                    fileStream.Close();
                }
            }
        }

        public static string GetSaveFileWithDialog()
        {
            var dialog = new SaveFileDialog() { Title = "Select a file to write the 4QL module to", DefaultExt = ".4ql", Filter = "4QL Files (.4ql)|*.4ql", AddExtension = true, OverwritePrompt = true, CheckPathExists = true };
            var result = dialog.ShowDialog();
            return result.HasValue && result == true ? dialog.FileName : null;
        }

        public static void SaveFile(string fileName, string content)
        {
            using (var fileStream = new FileStream(fileName, FileMode.Create))
            {
                using (var writer = new StreamWriter(fileStream))
                {
                    writer.Write(content);
                    writer.Close();
                }
                fileStream.Close();
            }
        }
    }
}
