﻿using FourqlModeler.Calculcations;
using FourqlModeler.Commands;
using FourqlModeler.Communication;
using FourqlModeler.Communication.Import;
using FourqlModeler.ViewModels;
using FourqlParser.Parsers;
using FourqlParser.Structures.Print;
using Microsoft.Win32;
using System;
using System.IO;
using System.Text;
using System.Windows;

namespace FourqlModeler.Dialogs
{
    /// <summary>
    /// Interaction logic for SimpleDialog.xaml
    /// </summary>
    public partial class SimpleDialog : Window
    {
        private string DefaultValue { get; set; }

        public SimpleDialog()
        {
            InitializeComponent();

            this.DefaultValue = this.CommandBox.Text;
        }

        void CommandBox_IsKeyboardFocusChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if ((Boolean) e.NewValue)
            {
                if (this.DefaultValue.Equals(this.CommandBox.Text))
                {
                    this.CommandBox.Text = String.Empty;
                }
            }
            else if (this.CommandBox.Text.Equals(String.Empty))
            {
                this.CommandBox.Text = this.DefaultValue;
            }
        }
        
        private void Send_Clear(object sender, RoutedEventArgs e)
        {
            IsEnabled = false;
            ApplicationClient.Instance.ClearModules();
            //ApplicationClient.Instance.SendCommand(MessageFactory.clearMessage(new SuccessHandler()));
            IsEnabled = true;
        }

        private void Send_Quit(object sender, RoutedEventArgs e)
        {
            IsEnabled = false;
           // ApplicationClient.Instance.SendCommand(MessageFactory.quitMessage(new SuccessHandler()));
            IsEnabled = true;
        }

        private void Send_Import(object sender, RoutedEventArgs e)
        {
            IsEnabled = false;
            var dialog = new OpenFileDialog();
            dialog.DefaultExt = ".4ql";
            dialog.Filter = "4QL modules (.4ql)|*.4ql";

            Nullable<bool> result = dialog.ShowDialog();
            if (result != null && !String.IsNullOrEmpty(dialog.FileName))
            {
                try
                {
                    new ImportExecutor().execute(dialog.FileName, ModuleRepository.Instance.Modules);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Import error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            IsEnabled = true;
        }

        private void Send_Rule(object sender, RoutedEventArgs e)
        {
            IsEnabled = false;
            try
            {
                var result = new CalcExprExecutor().Execute(this.CommandBox.Text, null, ModuleRepository.Instance.Modules);
                MainWindowViewModel.Instance.ConsoleViewModel.SendCommand(result.ToString());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Rule error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            IsEnabled = true;
        }
    }
}
