﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xceed.Wpf.AvalonDock.Layout;

namespace FourqlModeler.TabControl
{
    public class TabController
    {
        public IDictionary<string, LayoutDocument> OpenTabs { get; private set; }
        public LayoutPanel TabPanelParent { get; private set; }
        public LayoutDocumentPane TabPanel { get; private set; }

        public TabController(LayoutPanel tabPanelParent, LayoutDocumentPane tabPanel)
        {
            OpenTabs = new Dictionary<string, LayoutDocument>();
            this.TabPanelParent = tabPanelParent;
            this.TabPanel = tabPanel;
        }

        public LayoutDocument GetTab(string tabName)
        {
            return OpenTabs.ContainsKey(tabName) ? OpenTabs[tabName] : null;
        }

        public LayoutDocument createTab(string tabName, object content)
        {
            var document = new LayoutDocument() { Title = tabName, ContentId = tabName, Content = content };  
            TabPanel.Children.Add(document);

            document.Closed += closeTab;
            OpenTabs[tabName] = document;

            if (TabPanel.Parent == null)
            {
                // Re-add the panel if necessary.
                TabPanelParent.Children.Insert(0, TabPanel);
            }

            return document;
        }

        public LayoutDocument createTabIOrReturnExisting(string tabName, object content)
        {
            var document = GetTab(tabName);
            if (document != null)
            {
                return document;
            }

            return createTab(tabName, content);
        }

        public void selectTab(string tabName)
        {
            var document = GetTab(tabName);
            if (document != null)
            {
                selectTab(document);
            }
        }

        public void selectTab(LayoutDocument document)
        {
            if (!document.IsFloating)
            {
                var parent = document.Parent as LayoutDocumentPane;
                parent.SelectedContentIndex = parent.Children.IndexOf(document);
            }
        }

        public LayoutDocument getFocusedTab()
        {
            foreach (var entry in OpenTabs)
            {
                if (entry.Value.IsLastFocusedDocument)
                {
                    return entry.Value;
                }
            }
            return null;
        }

        void closeTab(object sender, EventArgs args)
        {
            OpenTabs.Remove((sender as LayoutDocument).ContentId);
        }
    }
}
