﻿using FourqlModeler.Calculcations;
using FourqlModeler.Views.ViewInterfaces;
using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using System.Windows.Input;
using FourqlModeler.Commands.Console;

namespace FourqlModeler.ViewModels
{
    class ConsoleViewModel : BaseViewModel, IConsoleView
    {

        private bool changeSwitch = false;
        public bool ChangeSwitch { get { return changeSwitch; } set { changeSwitch = value; RaisePropertyChanged("ChangeSwitch"); } }
        private String content = String.Empty;
        public String Content { get { return content; } set { content = value; RaisePropertyChanged("Content"); } }
        private int limitToLines = 0;
        public int LimitToLines { get { return limitToLines; } set { limitToLines = value; RaisePropertyChanged("LimitToLines"); } }

        public ICommand ExecuteExpressionCommand { get; set; }
        public ICommand ClearConsoleCommand { get; set; }
        public ICommand LimitConsoleToLinesCommand { get; set; }

        public static bool Debug { get; set; }

        public ConsoleViewModel()
        {
            MainWindowViewModel.Instance.ConsoleViewModel = this;
            ExecuteExpressionCommand = new ExecuteExpressionCommand();
            ClearConsoleCommand = new ClearConsoleCommand(this);
            LimitConsoleToLinesCommand = new LimitConsoleToLinesCommand(this);
        }

        public void SendCommand(string input)
        {
            UpdateConsole(Content + String.Format("[{0}] {1}\n", DateTime.Now.ToString(), input));
        }

        public void SendDebugCommand(string input)
        {
            if (Debug)
            {
                UpdateConsole(Content + String.Format("[{0}::Debug] {1}\n", DateTime.Now.ToString(), input));
            }
        }

        public void Clear()
        {
            UpdateConsole("");
        }

        public void LimitConsoleToLines(int lines)
        {
            LimitToLines = lines;
            UpdateConsole(Content);
        }

        private void UpdateConsole(string content)
        {
            if (LimitToLines > 0)
            {
                var builder = new StringBuilder();
                var lines = content.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                var startingLine = Math.Max(0, lines.Length - LimitToLines);
                for (int i = startingLine; i < lines.Length; ++i)
                {
                    builder.AppendLine(lines[i]);
                }
                App.Current.Dispatcher.Invoke(() => Content = builder.ToString());
            }
            else
            {
                App.Current.Dispatcher.Invoke(() => Content = content);
            }
            App.Current.Dispatcher.Invoke(() => ChangeSwitch = !ChangeSwitch);
        }
    }
}
