﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Structures;
using FourQLInterpreter.Reasoning;
using FourQLInterpreter.WSM;

namespace FourqlModeler.ViewModels
{
    internal class HistoryViewModel : BaseViewModel
    {
        private Module _module;

        public Module Module
        {
            get { return _module; }
            set
            {
                _module = value;
                RaisePropertyChanged("Module");
            }
        }

        private ObservableCollection<FactEvent> _events;

        public ObservableCollection<FactEvent> Events
        {
            get { return _events; }
            set
            {
                _events = value;
                RaisePropertyChanged("Events");
            }
        }


        public static ObservableCollection<FactEvent> BuildEvents(EvaluationHistory history)
        {

            var res = from it in history.Iterations
                from ev in it.Value
                from f in ev.Facts.DefaultIfEmpty()
                where f != null
                select new FactEvent()
                {
                    Fact = f == null ? null : f.Item1,
                    FactResult = f == null ? FactResult.NO_CHANGE : f.Item2,
                    Event = ev,
                    IterationNumber = it.Key,
                    PhaseName = ev.Type.ToString()
                };

            var col = new ObservableCollection<FactEvent>(res);

            AddModelStates(col);

            return col;
        }

        private static void AddModelStates(ObservableCollection<FactEvent> col)
        {
            StringBuilder sb = new StringBuilder();
            var l = 0;
            var maxLen = 80;
            var prevPhase = EventType.Retract;
            foreach (var factEv in col)
            {
                if (prevPhase != factEv.Event.Type && factEv.Event.Type == EventType.Retract)
                {
                    sb.Clear();
                }
                prevPhase = factEv.Event.Type;

                AddModelState(factEv, sb);
                if (sb.Length - l >= maxLen)
                {
                    l = sb.Length;
                    sb.AppendLine();
                }
            }
        }

        private static void AddModelState(FactEvent factEv, StringBuilder sb)
        {
            if (factEv.Event.Type != EventType.Retract)
                // we don't need any info for the retract phase
                factEv.ModelState = sb.ToString();
           

            if (!factEv.FactResult.Equals(FactResult.NO_CHANGE) || (factEv.Event.Type == EventType.Retract))
            {
                sb.Append(factEv.Fact + "  ");
            }
        }
    }

    class EvalEvent : BaseViewModel
    {

        private string _name;

        public string PhaseName { get { return _name; } set { _name = value; RaisePropertyChanged("PhaseName"); } }

        private int _number;

        public int IterationNumber { get { return _number; } set { _number = value; RaisePropertyChanged("IterationNumber"); } }

        private EvaluationEvent _event;

        public EvaluationEvent Event { get { return _event; } set { _event = value; RaisePropertyChanged("Event"); } } 
    }

    class FactEvent : EvalEvent
    {
        private Fact _fact;

        public Fact Fact { get { return _fact; } set { _fact = value; RaisePropertyChanged("Fact"); } }

        private FactResult _result;
        public FactResult FactResult { get { return _result; } set { _result = value; RaisePropertyChanged("FactResult"); } }

        private string _modelState;

        public string ModelState { get { return _modelState; } set { _modelState = value; RaisePropertyChanged("ModelState"); } }

    }


}
