﻿using FourqlModeler.Commands;
using FourqlModeler.Dialogs;
using FourqlModeler.Utility;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using FourqlParser.Structures;
using FourqlModeler.Commands.Tools;

namespace FourqlModeler.ViewModels
{
    public class ModuleStorageViewModel : BaseViewModel
    {
        private ObservableCollection<ModuleViewModel> modules = new ObservableCollection<ModuleViewModel>();
        public ObservableCollection<ModuleViewModel> Modules { get { return modules; } set { modules = value; RaisePropertyChanged("Modules"); } }

        private ObservableCollection<ErrorDesc> allErrors = new ObservableCollection<ErrorDesc>();
        public ObservableCollection<ErrorDesc> AllErrors { get { return allErrors; } set { allErrors = value; RaisePropertyChanged("AllErrors"); } }

        private ErrorDesc selectedError = null;
        public ErrorDesc SelectedError { get { return selectedError; } set { selectedError = value; RaisePropertyChanged("SelectedError"); } }

        public ICommand BuildModulesCommand { get; set; }
        public ICommand BuildAllModulesCommand { get; set; }
        public ICommand ShowModuleGraphCommand { get; set; }
        public ICommand NewModuleCommand { get; set; }
        public ICommand NewSampleModuleCommand { get; set; }
        public ICommand LoadModulesCommand { get; set; }
        public ICommand UnloadModuleCommand { get; set; }
        public ICommand UnloadFolderModulesCommand { get; set; }
        public ICommand UnloadAllModulesCommand { get; set; }
        public ICommand RestoreModuleCommand { get; set; }
        public ICommand SaveModuleCommand { get; set; }
        public ICommand SaveAllModulesCommand { get; set; }
        public ICommand SettingsCommand { get; set; }
        public ICommand HelpCommand { get; set; }
        public ICommand AboutCommand { get; set; }
        public ICommand ExitCommand { get; set; }
        public ICommand OpenModuleCommand { get; set; }
        public ICommand SaveFocusedModuleCommand { get; set; }
        public ICommand ShowErrorSpotCommand { get; set; }
        public ICommand TestRefreshCommand { get; set; }        

        public ModuleStorageViewModel()
        {
            BuildModulesCommand = new BuildModulesCommand(this);
            BuildAllModulesCommand = new BuildAllModulesCommand(this);
            ShowModuleGraphCommand = new ShowModuleGraphCommand(this);
            NewModuleCommand = new NewModuleCommand(this);
            NewSampleModuleCommand = new NewSampleModuleCommand(this);
            LoadModulesCommand = new LoadModulesCommand(this);
            UnloadModuleCommand = new UnloadModuleCommand(this);
            UnloadFolderModulesCommand = new UnloadFolderModulesCommand(this);
            UnloadAllModulesCommand = new UnloadAllModulesCommand(this);
            RestoreModuleCommand = new RestoreModuleCommand(this);
            SaveModuleCommand = new SaveModuleCommand(this);
            SaveAllModulesCommand = new SaveAllModulesCommand(this);
            SettingsCommand = new NotImplementedCommand();
            HelpCommand = new NotImplementedCommand();
            AboutCommand = new AboutCommand();
            ExitCommand = new ExitCommand();

            OpenModuleCommand = new OpenModuleCommand();
            SaveFocusedModuleCommand = new SaveFocusedModuleCommand(this);
            ShowErrorSpotCommand = new ShowErrorSpotCommand(this);

            TestRefreshCommand = new TestRefreshCommand();
        }
    }

    public class ModuleViewModel : BaseViewModel
    {
        public static readonly string InternalExternalDefinition = "\\Internal+External\\";

        public bool IsInternalExternal { get { return FileDir.Equals(InternalExternalDefinition); } }

        private string fileName = String.Empty;
        public String FileName { get { return fileName; } set { fileName = value; RaisePropertyChanged("FileName"); } }

        private string fileDir = String.Empty;
        public String FileDir { get { return fileDir; } set { fileDir = value; RaisePropertyChanged("FileDir"); RaisePropertyChanged("IsStatic"); } }

        private string savedContent = String.Empty;
        public String SavedContent { get { return savedContent; } set { savedContent = value; RaisePropertyChanged("SavedContent"); } }

        private string content = String.Empty;
        public String Content { get { return content; } set { content = value; RaisePropertyChanged("Content"); IsDirty = !content.Equals(savedContent); } }

        private bool isDirty = false;
        public bool IsDirty { get { return isDirty; } set { isDirty = value; RaisePropertyChanged("IsDirty"); } }

        private ObservableCollection<ErrorDesc> syntaxErrors = new ObservableCollection<ErrorDesc>();
        public ObservableCollection<ErrorDesc> SyntaxErrors { get { return syntaxErrors; } set { syntaxErrors = value; RaisePropertyChanged("SyntaxErrors"); } }

        private ObservableCollection<ErrorDesc> staticAnalysisErrors = new ObservableCollection<ErrorDesc>();
        public ObservableCollection<ErrorDesc> StaticAnalysisErrors { get { return staticAnalysisErrors; } set { staticAnalysisErrors = value; RaisePropertyChanged("StaticAnalysisErrors"); } }

        private IDictionary<string, ObservableCollection<ErrorDesc>> moduleStaticAnalysisErrors = new Dictionary<string, ObservableCollection<ErrorDesc>>();
        public IDictionary<string, ObservableCollection<ErrorDesc>> ModuleStaticAnalysisErrors { get { return moduleStaticAnalysisErrors; } set { moduleStaticAnalysisErrors = value; RaisePropertyChanged("ModuleStaticAnalysisErrors"); } }
        
        private Program loadedProgram;
        public Program LoadedProgram { get { return loadedProgram; } set { loadedProgram = value; RaisePropertyChanged("LoadedProgram"); } }

        private Program currentProgram;
        public Program CurrentProgram { get { return currentProgram; } set { currentProgram = value; RaisePropertyChanged("CurrentProgram"); } }
    }

    public class ErrorDesc
    {
        public int Id { get; set; }
        public int GroupId { get; set; }
        public ModuleViewModel ModelSource { get; set; }
        public String ModuleName { get; set; }
        public int StartLine { get; set; }
        public int EndLine { get; set; }
        public int StartColumn { get; set; }
        public int EndColumn { get; set; }
        public string Category { get; set; }
        public string Description { get; set; }
    }
}
