﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using FourqlParser.Structures;

namespace FourqlModeler.Views.GraphExtensions.ExprGraph
{
    public class ExplanationBrushes
    {
        public static Brush SelectedBrush = new SolidColorBrush(Colors.LightGreen);
        public static Brush RootBrush = new SolidColorBrush(Colors.Red);
        public static Brush DependencyBrush = new SolidColorBrush(Colors.Blue);
        public static Brush NoAction = null;
    }

    public class LogicValueVertex : ExtendedVertex
    {
        public LogicValue? LogicValue { get; set; }
        public string LogicValueString { get { return LogicValue == null ? "?" : LogicValue.ToString(); } }
    }

    public class PatternVertex : LogicValueVertex
    {
        public string Pattern { get; set; }
    }

    public class OperationVertex : PatternVertex
    {
        public string Operation { get; set; }
    }

    public class RuleAlternativeVertex : OperationVertex
    {
        public RuleAlternativeVertex()
        {
            Operation = "RA";
        }
    }

    public class ModuleVertex : OperationVertex
    {
        public string ModuleName { get; set; }

        public ModuleVertex()
        {
            Operation = "M";
        }
    }

    public class LiteralVertex : ModuleVertex
    {
        public Literal Literal { get; set; }


        public LiteralVertex()
        {
            Operation = "L";
        }
    }

    public class NegVertex : OperationVertex
    {
        public NegVertex()
        {
            Operation = "N";
        }
    }

    public class AndVertex : OperationVertex
    {
        public AndVertex()
        {
            Operation = "A";
        }
    }

    public class OrVertex : OperationVertex
    {
        public OrVertex()
        {
            Operation = "O";
        }
    }

    public class InVertex : OperationVertex
    {
        public InVertex()
        {
            Operation = "In";
        }
    }

    public class QuantifierVariableVertex : OperationVertex
    {
        public String VariableName { get; set; }
        public String VariableDomain { get; set; }
        public object VariableValue { get; set; }

        public QuantifierVertex Parent { get; private set; }
        public QuantifierVariableVertex(QuantifierVertex parent)
        {
            Operation = "V";
            this.VariableDomain = parent.DomainModule + "." + parent.VariableDomain;
            this.VariableName = parent.VariableName;
            this.VariableValue = parent.VariableValue == null ? null : parent.VariableValue.Value;
            this.Pattern = MakePattern(VariableName, parent.DomainModule + "." + parent.VariableDomain, VariableValue);
            this.Parent = parent;
            this.LogicValue = parent.LogicValue;
            this.Status = ExtendedVertexStatus.VariableSelection;
        }

        public static String MakePattern(String variableName, String variableDomain, object variableValue)
        {
            return variableName + ":" + variableDomain + " = " + (variableValue == null ? "?" : variableValue.ToString());
        }
    }

    public class QuantifierVertex : OperationVertex
    {
        public String VariableName { get; set; }
        public ValueHolder VariableValue { get; set; }
        public string VariableDomain { get; set; }
        public string DomainModule { get; set; }
        public Expression SubExpression { get; set; }
        public String ModuleName { get; set; }
    }

    public class ExistsVertex : QuantifierVertex
    {
        public ExistsVertex ()
        {
            Operation = "Ex";
        }
    }

    public class AllVertex : QuantifierVertex
    {
        public AllVertex()
        {
            Operation = "All";
        }
    }
}
