﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Data;
using FourqlModeler.ViewModels;
using FourQLInterpreter.Reasoning;
using FourQLInterpreter.WSM;

namespace FourqlModeler.Views.ValueConverters
{
   public  class IterationResultConverter : IValueConverter
    {
            public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                
                var facts = new List<FactEvent>();
                ExtractObjects<FactEvent>(value, facts);
                var maxLen = 120;
                var l = 0;
                IEnumerable<FactEvent> filteredFacts;
                if (facts.Any(f => f.Event.Type.Equals(EventType.Retract)))
                {
                    // this iteration had a retract phase => show what's left
                    filteredFacts = from factEvent in facts
                        where factEvent.Event.Type.Equals(EventType.Retract)
                        select factEvent;
                }
                else
                {
                    // this iteration didn't have a retract phase => it's the last iteration, so just show everything
                    filteredFacts = facts;
                }

                StringBuilder sb = new StringBuilder();
                //sb.AppendLine();

                var strings = new SortedSet<string>();

                foreach (var fact in filteredFacts)
                {
                    if (fact.Fact != null)
                        strings.Add(fact.Fact.ToString());
                }
                foreach (var s in strings)
                {
                    sb.Append(s + "  ");
                    if (sb.Length - l >= maxLen)
                    {
                        l = sb.Length;
                        sb.AppendLine();
                    }
                }
                return sb.ToString();
            }

            public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                throw new NotSupportedException();
            }

       public static void ExtractObjects<T>(object obj, ICollection<T> result) where T : class
       {
           var extracted = obj as T;
            if (extracted != null)
            {
                result.Add(extracted);
                return;
            }

           var simpleColl = obj as ICollection<object>;
           if (simpleColl != null)
           {
               foreach (var elem in simpleColl)
               {
                   ExtractObjects<T>(elem, result);
               }
               return;
           }

           var groupView = obj as CollectionViewGroup;
           if (groupView != null)
           {
               foreach (var item in groupView.Items)
               {
                   ExtractObjects<T>(item, result);
               }
           }
       }

       
   }

}
