﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using FourqlModeler.ViewModels;
using FourQLInterpreter.Reasoning;
using FourQLInterpreter.WSM;

namespace FourqlModeler.Views.ValueConverters
{
    class PhaseTitleConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return value.Equals("Retract") ? "Inconsistent facts left after the phase:" : "Facts added to the model in this phase:";
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }

    class PhaseResultConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var facts = new List<FactEvent>();
            IterationResultConverter.ExtractObjects<FactEvent>(value, facts);
            var maxLen = 120;
            var l = 0;
            IEnumerable<FactEvent> filteredFacts;
            

            StringBuilder sb = new StringBuilder();

            if (facts.Any(f => f.Event.Type.Equals(EventType.Retract)))
            {
                filteredFacts = facts.Where(f => f.FactResult.Equals(FactResult.NO_CHANGE));
                //sb.Append(" Facts left after the phase:");
            }
            else
            {
                filteredFacts = facts.Where(f => !f.FactResult.Equals(FactResult.NO_CHANGE));
                //sb.Append(" Facts added to the model in this phase:");
            }

            //sb.AppendLine();
            var strings = new SortedSet<string>();

            foreach (var fact in filteredFacts)
            {
                if (fact.Fact != null)
                    strings.Add(fact.Fact.ToString());
            }
            foreach (var s in strings)
            {
                sb.Append(s + "  ");
                if (sb.Length - l >= maxLen)
                {
                    l = sb.Length;
                    sb.AppendLine();
                }
            }
            return sb.ToString();
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}
