﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using FourqlModeler.Calculcations;
using FourqlModeler.ViewModels;
using FourqlParser.Structures;

namespace FourqlModeler.Views.ValueConverters
{
    class RuleSubstitutionTooltipConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var factEv = value as FactEvent;
            if (factEv.Fact != null && factEv.Event.Rule != null)
            {
                var ruleWithReplacements = BuildRuleWithReplacements(factEv.Fact, factEv.Event.Rule);

                return "Fact was deduced from rule:" +
                       "\n\t" + factEv.Event.Rule + "\n" +
                       "With substitutions:" +
                       "\n\t" + ruleWithReplacements;
            }
            return "";
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }


        private Rule BuildRuleWithReplacements(Fact fact, Rule rule)
        {
            var values = new Dictionary<string, ValueHolder>();
            var literalIter = fact.Values.GetEnumerator();
            var headIter = rule.Head.Values.GetEnumerator();
            while (literalIter.MoveNext() && headIter.MoveNext())
            {
                if (headIter.Current.Type == DomainType.VARIABLE && !values.ContainsKey(headIter.Current.valueAs<string>()))
                {
                    values.Add(headIter.Current.valueAs<string>(), literalIter.Current);
                }
            }
            var setter = new ExprValueSetter(values);
            return new Rule()
            {
                ParserContext = rule.ParserContext, 
                Head = setter.SetInFact(rule.Head), 
                Body = rule.Body == null ? null : rule.Body.Accept(setter)
            };

        }
    }
}
