﻿using FourqlParser.Structures;
using FourqlParser.Structures.Optimization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Parsers.ExpParser.Converters
{
    class ComplexExprConverter : ExprParserBaseVisitor<Expression>
    {
        private Literal Literal;
        private ModuleExpression ModuleToNegate = null;
        private ModuleExpression Module;

        public Expression convert(ExprParser.ExpressionRuleContext context)
        {
            ModuleToNegate = null;
            return new ExpressionOptimizer().optimize(context.Accept(this));
        }

        #region IExprParserVisitor<Expression> Members

        public override Expression VisitExpr(ExprParser.ExprContext context)
        {
            ModuleToNegate = null;
            return context.Expr.Accept(this);
        }

        public override Expression VisitOrExpr(ExprParser.OrExprContext context)
        {
            var expr = new OrExpression() { ParserContext = context };
            expr.Nexts.Add(context.Head.Accept(this));
            if (context.Tail != null)
            {
                ModuleToNegate = null;
                expr.Nexts.Add(context.Tail.Accept(this));
            }
            return expr;
        }

        public override Expression VisitAndExpr(ExprParser.AndExprContext context)
        {
            var expr = new AndExpression() { ParserContext = context };
            expr.Nexts.Add(context.Head.Accept(this));
            if (context.Tail != null)
            {
                ModuleToNegate = null;
                expr.Nexts.Add(context.Tail.Accept(this));
            }
            return expr;
        }

        public override Expression VisitNonNegExpr(ExprParser.NonNegExprContext context)
        {
            return context.Expr.Accept(this);
        }

        public override Expression VisitNegExpr(ExprParser.NegExprContext context)
        {
            var next = context.Expr.Accept(this);
            var nextAsModule = next as ModuleExpression;
            if (nextAsModule != null)
            {
                nextAsModule.IsNegative = true;
                return next;
            }
            return new NegExpression() { Next = next, ParserContext = context };
        }
        
        public override Expression VisitParentExpr(ExprParser.ParentExprContext context)
        {
            return context.Expr.Accept(this);
        }

        public override Expression VisitModuleExpr(ExprParser.ModuleExprContext context)
        {
            var moduleBackup = Module;
            Module = new ModuleExpression() { ModuleName = context.ModuleName.Text, Next = context.Expr.Accept(this), ParserContext = context };
            if (context.InList != null)
            {
                context.InList.Accept(this);
            }
            var module = Module;
            Module = moduleBackup;
            return module;
        }

        public override Expression VisitLiteralExpr(ExprParser.LiteralExprContext context)
        {
            return context.Literal.Accept(this);
        }

        public override Expression VisitLiteral(ExprParser.LiteralContext context)
        {
            Literal = new Literal() { RelationName = context.RelationName.Text, ParserContext = context };
            context.LiteralArgs.Accept(this);
            return Literal;
        }

        public override Expression VisitLiteralVariableArgument(ExprParser.LiteralVariableArgumentContext context)
        {
            Literal.Values.Add(new ValueHolder(context.VariableName.Text, DomainType.VARIABLE));
            return Literal;
        }

        public override Expression VisitLiteralArguments(ExprParser.LiteralArgumentsContext context)
        {
            context.Head.Accept(this);
            if (context.Tail != null)
            {
                context.Tail.Accept(this);
            }
            return Literal;
        }

        public override Expression VisitInLogicArguments(ExprParser.InLogicArgumentsContext context)
        {
            Module.InValues.Add(context.Head.Accept(new LogicValueConverter()));
            if (context.Tail != null)
            {
                context.Tail.Accept(this);
            }
            return Module;
        }

        public override Expression VisitSingleInLogicArgument(ExprParser.SingleInLogicArgumentContext context)
        {
            Module.InValues.Add(context.Head.Accept(new LogicValueConverter()));
            return Module;
        }

        public override Expression VisitLiteralValueArgument(ExprParser.LiteralValueArgumentContext context)
        {
            Literal.Values.Add(context.Value.Accept(new ValueConverter()));
            return Literal;
        }

        public override Expression VisitNoLiteralArguments(ExprParser.NoLiteralArgumentsContext context)
        {
            return Literal;
        }

        #endregion
    }
}
