﻿using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Parsers.ExpParser.Converters
{
    class ValueConverter : ExprParserBaseVisitor<ValueHolder>
    {
        public override ValueHolder VisitValueLogic(ExprParser.ValueLogicContext context)
        {
            return new ValueHolder(context.LogicValue.Accept(new LogicValueConverter()), DomainType.LOGIC);
        }

        public override ValueHolder VisitValueLiteral(ExprParser.ValueLiteralContext context)
        {
            return new ValueHolder(context.LiteralValue.Text, DomainType.LITERAL);
        }

        public override ValueHolder VisitValueString(ExprParser.ValueStringContext context)
        {
            var value = context.StringValue.Text;
            return new ValueHolder(value.Substring(1, value.Length - 2), DomainType.STRING);
        }

        public override ValueHolder VisitValueInteger(ExprParser.ValueIntegerContext context)
        {
            return new ValueHolder(Int32.Parse(context.IntegerValue.Text), DomainType.INTEGER);
        }

        public override ValueHolder VisitValueReal(ExprParser.ValueRealContext context)
        {
            return new ValueHolder(Double.Parse(context.RealValue.Text, CultureInfo.InvariantCulture), DomainType.REAL);
        }

        public override ValueHolder VisitValueDate(ExprParser.ValueDateContext context)
        {
            return new ValueHolder(DateTime.ParseExact(context.DateValue.Text, "dd-MM-yyyy", CultureInfo.InvariantCulture), DomainType.DATE);
        }

        public override ValueHolder VisitValueDateTime(ExprParser.ValueDateTimeContext context)
        {
            return new ValueHolder(DateTime.ParseExact(context.DateTimeValue.Text, "dd-MM-yyyy HH:mm", CultureInfo.InvariantCulture), DomainType.DATE_TIME);
        }
    }
}
