parser grammar ExprParser;

options {
	tokenVocab = ModuleLexer;
}


/*
 * Parser Rules
 */
 expressionRule
	: Expr=alternativeRule DOT EOF									# Expr
	;

alternativeRule
	: Head=conjunctionRule (ALTERNATIVE Tail=alternativeRule)?		# OrExpr
	;

conjunctionRule
	: Head=negExprRule (COMMA Tail=conjunctionRule)?				# AndExpr
	;

negExprRule
	: NEG_SIGN Expr=unaryExprRule									# NegExpr
	| Expr=unaryExprRule											# NonNegExpr
	;

unaryExprRule
	: OPENING_PARENT Expr=alternativeRule CLOSING_PARENT			# ParentExpr
	| ModuleName=IDENT DOT Expr=unaryExprRule (IN OPENING_CURLY InList=inLogicArgsRule CLOSING_CURLY)? # ModuleExpr
	| Literal=literalRule											# LiteralExpr
	;

inLogicArgsRule
	: Head=logicValueRule											# SingleInLogicArgument
	| Head=logicValueRule COMMA Tail=inLogicArgsRule				# InLogicArguments
	;

literalRule
	: RelationName=IDENT OPENING_PARENT LiteralArgs=literalArgsRule CLOSING_PARENT # Literal
	;

literalArgsRule
	: Head=literalArgRule (COMMA Tail=literalArgsRule)?				# LiteralArguments
	| /* Empty */													# NoLiteralArguments
	;

literalArgRule
	: VariableName=VARIABLE											# LiteralVariableArgument
	| Value=valueRule												# LiteralValueArgument
	;

// Values - used in facts and rule literals.
valueRule
	: LogicValue=logicValueRule										# ValueLogic
	| LiteralValue=IDENT											# ValueLiteral
	| StringValue=VALUE_STRING										# ValueString
	| IntegerValue=VALUE_INTEGER									# ValueInteger
	| RealValue=VALUE_REAL											# ValueReal
	| DateValue=VALUE_DATE											# ValueDate
	| DateTimeValue=VALUE_DATETIME									# ValueDateTime
	;

logicValueRule
	: VALUE_TRUE													# ValueTrue
	| VALUE_FALSE													# ValueFalse
	| VALUE_INCONS													# ValueInconsistent
	| VALUE_UNKNOWN													# ValueUnknown
	;
