﻿using FourqlParser.Structures;
using FourqlParser.Structures.Optimization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Parsers.ModParser.Converters
{
    class ExprConverter : ModuleParserBaseVisitor<Expression>
    {
        public MultiExpression CurrentOrExpr = null;
        public MultiExpression CurrentAndExpr = null;

        public Expression convert(ModuleParser.RuleBodyRuleContext context)
        {
            return new ExpressionOptimizer().optimize(context.Accept(this));
        }

        #region IModuleParserVisitor<Expression> Members

        public override Expression VisitRuleAlternative(ModuleParser.RuleAlternativeContext context)
        {
            if (CurrentOrExpr == null)
            {
                CurrentOrExpr = new OrExpression() { ParserContext = context };
            }

            CurrentAndExpr = null;
            CurrentOrExpr.Nexts.Add(context.Head.Accept(this));
            if (context.Tail != null)
            {
                context.Tail.Accept(this);
            }
            return CurrentOrExpr;
        }

        public override Expression VisitRuleConjunction(ModuleParser.RuleConjunctionContext context)
        {
            if (CurrentAndExpr == null)
            {
                CurrentAndExpr = new AndExpression() { ParserContext = context.Head };
            }

            CurrentAndExpr.Nexts.Add(context.Head.Accept(this));
            if (context.Tail != null)
            {
                context.Tail.Accept(this);
            }
            return CurrentAndExpr;
        }

        public override Expression VisitRuleLiteral(ModuleParser.RuleLiteralContext context)
        {
            var expr = context.Literal.Accept(new LiteralConverter());
            if (context.IsNegative != null)
            {
                return new NegExpression() { Next = expr, ParserContext = context };
            }
            return expr;
        }

        public override Expression VisitModuleRuleLiteral(ModuleParser.ModuleRuleLiteralContext context)
        {
            return new ModuleExpression() { ModuleName = context.ModuleName.Text, IsNegative = context.IsNegative != null, Next = context.Literal.Accept(new LiteralConverter()), InValues = context.InList != null ? context.InList.Accept(new InConverter()) : new List<LogicValue>(), ParserContext = context };
        }

        #endregion
    }
}
