lexer grammar ModuleLexer;
	

/*
 * Lexer Rules
 */
 // Comments.
COMMENT
    : ('/*' (.)*? '*/' | '//' ~('\r' | '\n')*) -> channel(HIDDEN)
    ;

// Keywords - module (+end), sections (domains, relations, rules, facts).
MODULE
	: 'module'
	;

END
	: 'end'
	;

DOMAINS
	: 'domains'
	;

RELATIONS
	: 'relations'
	;

RULES
	: 'rules'
	;

FACTS
	: 'facts'
	;

// Separators and special signs.
IS_CONCLUSION_OF
	: ':-'
	;

COLON
	: ':'
	;

ALTERNATIVE
	: '|'
	;

DOT
	: '.'
	;

COMMA
	: ','
	;

OPENING_PARENT
	: '('
	;

CLOSING_PARENT
	: ')'
	;

OPENING_CURLY
	: '{'
	;

CLOSING_CURLY
	: '}'
	;

IN
	: 'in'
	;

// Type tokens.
TYPE_LOGIC
	: 'logic'
	;

TYPE_LITERAL
	: 'literal'
	;

TYPE_STRING
	: 'string'
	;

TYPE_INTEGER
	: 'integer'
	;

TYPE_REAL
	: 'real'
	;

TYPE_DATE
	: 'date'
	;

TYPE_DATETIME
	: 'date_time'
	;

// Value tokens.
VALUE_TRUE
	: 'true'
	;

VALUE_FALSE
	: 'false'
	;

VALUE_INCONS
	: 'inconsistent'
	| 'incons'
	;

VALUE_UNKNOWN
	: 'unknown'
	;

VALUE_STRING
	: '"' SEQ_CHAR* '"'
	;

VALUE_INTEGER
	: NEG_SIGN? DIGIT+
	;

VALUE_REAL
	: NEG_SIGN? DIGIT+ '.' DIGIT*
	;

VALUE_DATE
	: [0-3][0-9] '-' [0-1][0-9] '-' [1-2][0-9][0-9][0-9]
	;

VALUE_DATETIME
	: [0-3][0-9] '-' [0-1][0-9] '-' [1-2][0-9][0-9][0-9] ' ' [0-2][0-9] ':' [0-5][0-9]
	;

// Other names in modules.
VARIABLE
	: CAPITAL_LETTER (LETTER | DIGIT | '_')*
	;

IDENT
	: (LETTER  | '_'+ (LETTER | DIGIT)) (LETTER | DIGIT | '_')*
	;

// Value-supporting tokens.
NEG_SIGN
	: '-'
	;

// Fragment tokens.
fragment
DIGIT
	: [0-9]
	;

fragment
SMALL_LETTER
	: [a-z]
	;

fragment
CAPITAL_LETTER
	: [A-Z]
	;

fragment
SEQ_CHAR
	: ~["\\\r\n]
	| '\\' ['"?abfnrtv\\]
	;

fragment
LETTER
	: SMALL_LETTER
	| CAPITAL_LETTER
	;

// White signs.
WHITE_SIGN
	: [ \t\r\n]+ -> channel(HIDDEN)
	;