parser grammar ModuleParser;

options {
	tokenVocab = ModuleLexer;
}


/*
 * Parser Rules
 */
programRule
	: Modules=moduleRule* EOF								# Program
	;

moduleRule
	: MODULE ModuleName=IDENT COLON Domains=domainsRule Relations=relationsRule Rules=rulesRule Facts=factsRule END DOT # Module
	;
	
// Domains:
domainsRule
	: DOMAINS COLON DomainList=domainRule*					# Domains
	| /* Empty */											# NoDomains
	;

domainRule
	: Type=typeRule DomainName=IDENT DOT					# Domain
	;

// Relations:
relationsRule
	: RELATIONS COLON RelationList=relationRule*			# Relations
	| /* Empty */											# NoRelations
	;

relationRule
	: RelationName=IDENT OPENING_PARENT RelationArgs=relationArgsRule CLOSING_PARENT DOT # Relation
	;

relationArgsRule
	: Head=relationArgRule (COMMA Tail=relationArgsRule)?	# RelationArguments
	| /* Empty */											# NoRelationArguments
	;

relationArgRule
	: DomainName=IDENT										# RelationDomainArg
	| Type=typeRule											# RelationTypeArg
	;

// Rules:
rulesRule
	: RULES COLON RuleList=ruleRule*						# Rules
	| /* Empty */											# NoRules
	;

ruleRule
	: Head=ruleHeadRule IS_CONCLUSION_OF Body=ruleBodyRule DOT	# RuleDef
	;

ruleHeadRule
	: (IsNegative=NEG_SIGN)? Literal=literalRule				# RuleHead
	;

ruleBodyRule
	: Head=ruleConjunctionRule (ALTERNATIVE Tail=ruleBodyRule)?	# RuleAlternative
	;

ruleConjunctionRule
	: Head=ruleLiteralRule (COMMA Tail=ruleConjunctionRule)?	# RuleConjunction
	;

// Minus wiaze mocniej niz IN, wiec jest wykonywany przed aplikacja IN.
ruleLiteralRule
	: (IsNegative=NEG_SIGN)? Literal=literalRule																				# RuleLiteral
	| (IsNegative=NEG_SIGN)? ModuleName=IDENT DOT Literal=literalRule (IN OPENING_CURLY InList=inLogicArgsRule CLOSING_CURLY)?	# ModuleRuleLiteral
	;

inLogicArgsRule
	: Head=logicValueRule									# SingleInLogicArgument
	| Head=logicValueRule COMMA Tail=inLogicArgsRule		# InLogicArguments
	;

literalRule
	: RelationName=IDENT OPENING_PARENT LiteralArgs=literalArgsRule CLOSING_PARENT # Literal
	;

literalArgsRule
	: Head=literalArgRule (COMMA Tail=literalArgsRule)?	# LiteralArguments
	| /* Empty */										# NoLiteralArguments
	;

literalArgRule
	: VariableName=VARIABLE						# LiteralVariableArgument
	| Value=valueRule							# LiteralValueArgument
	;

// Facts:
factsRule
	: FACTS COLON FactList=factRule*			# Facts
	| /* Empty */								# NoFacts
	;

factRule
	: (IsNegative=NEG_SIGN)? RelationName=IDENT OPENING_PARENT FactArgs=factArgsRule CLOSING_PARENT DOT # Fact
	;

factArgsRule
	: Head=valueRule (COMMA Tail=factArgsRule)?	# FactArguments
	| /* Empty */								# NoFactArguments
	;

// Values - used in facts and rule literals.
valueRule
	: LogicValue=logicValueRule					# ValueLogic
	| LiteralValue=IDENT						# ValueLiteral
	| StringValue=VALUE_STRING					# ValueString
	| IntegerValue=VALUE_INTEGER				# ValueInteger
	| RealValue=VALUE_REAL						# ValueReal
	| DateValue=VALUE_DATE						# ValueDate
	| DateTimeValue=VALUE_DATETIME				# ValueDateTime
	;

logicValueRule
	: VALUE_TRUE								# ValueTrue
	| VALUE_FALSE								# ValueFalse
	| VALUE_INCONS								# ValueInconsistent
	| VALUE_UNKNOWN								# ValueUnknown
	;

// Types - used in relation and domain definitions.
typeRule
	: TYPE_LOGIC								# TypeLogic
	| TYPE_LITERAL								# TypeLiteral
	| TYPE_STRING								# TypeString
	| TYPE_INTEGER								# TypeInteger
	| TYPE_REAL									# TypeReal
	| TYPE_DATE									# TypeDate
	| TYPE_DATETIME								# TypeDateTime
	;