﻿using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.StaticAnalysis
{
    class ExprVariableGatherer
    {
        public static IDictionary<String, ICollection<VariableRef>> findVariablesOfLiteral(Module module, Literal literal)
        {
            var variables = new Dictionary<string, ICollection<VariableRef>>();
            var relation = module.FindRelation(literal.RelationName);
            var paramNo = 0;

            foreach (var arg in literal.Values) {
                if (arg.Type == DomainType.VARIABLE)
                {
                    var variableRef = new VariableRef() { Module = module, Literal = literal, ParamNumber = paramNo, VariableName = arg.valueAs<string>() };
                    if (relation.PrimitiveDomains.ContainsKey(paramNo))
                    {
                        variableRef.DomainType = relation.PrimitiveDomains[paramNo];
                    }
                    else
                    {
                        var domain = module.FindDomain(relation.DefinedDomains[paramNo]);
                        variableRef.DomainName = domain.Name;
                        variableRef.DomainType = domain.Type;
                    }

                    ICollection<VariableRef> variablesOf;
                    if (variables.ContainsKey(variableRef.VariableName))
                    {
                        variablesOf = variables[variableRef.VariableName];
                    }
                    else
                    {
                        variablesOf = new List<VariableRef>();
                        variables[variableRef.VariableName] = variablesOf;
                    }
                    variablesOf.Add(variableRef);
                }
                ++paramNo;
            }
            return variables;
        }
    }

    class VariableRef {
        public Module Module { get; set; }
        public Literal Literal { get; set; }
        public int ParamNumber { get; set; }
        public String VariableName { get; set; }
        public String DomainName { get; set; }
        public DomainType DomainType { get; set; }
    }
}
