﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Structures;

namespace FourqlParser.StaticAnalysis
{
    public class ModuleInvalidDefinitions
    {
        public IDictionary<string, ICollection<Domain>> DuplicatedDomains { get; set; }
        public IDictionary<string, ICollection<Relation>> DuplicatedRelations { get; set; }
        public IDictionary<Relation, ICollection<InvalidDefinition>> InvalidRelations { get; set; }
        public IDictionary<Rule, ICollection<InvalidDefinition>> InvalidRules { get; set; }
        public IDictionary<Fact, ICollection<InvalidDefinition>> InvalidFacts { get; set; }

        public ModuleInvalidDefinitions()
        {
            DuplicatedDomains = new Dictionary<string, ICollection<Domain>>();
            DuplicatedRelations = new Dictionary<string, ICollection<Relation>>();
            InvalidRelations = new Dictionary<Relation, ICollection<InvalidDefinition>>();
            InvalidRules = new Dictionary<Rule, ICollection<InvalidDefinition>>();
            InvalidFacts = new Dictionary<Fact, ICollection<InvalidDefinition>>();
        }
        
        public void removeInvalidConstructions(Module module)
        {
            copyWithoutInvalidConstructions(module, module);
        }

        public Module createCopyWithoutInvalidConstructions(Module module)
        {
            var moduleCopy = new Module() { Name = module.Name, ParserContext = module.ParserContext };
            copyWithoutInvalidConstructions(module, moduleCopy);
            return moduleCopy;
        }

        public void copyWithoutInvalidConstructions(Module source, Module dest)
        {
            dest.Domains = source.Domains.Where(domain => !DuplicatedDomains.ContainsKey(domain.Name)).ToList();
            dest.Relations = source.Relations.Where(relation => !DuplicatedRelations.ContainsKey(relation.Name) && !InvalidRelations.ContainsKey(relation)).ToList();
            dest.Rules = source.Rules.Where(rule => !InvalidRules.ContainsKey(rule)).ToList();
            dest.Facts = source.Facts.Where(fact => !InvalidFacts.ContainsKey(fact)).ToList();
        }

        public int ErrorCount()
        {
            return DuplicatedDomains.Count + DuplicatedRelations.Count + InvalidRelations.Count + InvalidFacts.Count + InvalidRules.Count;
        }

        public int TotalErrorCount()
        {
            return DuplicatedDomains.Count + DuplicatedRelations.Count + InvalidRelations.Count + InvalidFacts.Count + InvalidRules.Count;
        }
    }

    public class InvalidDefinition
    {
        public IOutputSource ErrorSource { get; set; }
        public string Message { get; set; }
    }

    public class InvalidDefinitionUtils
    {
        public static void addErrorsWithDesc<K, V>(IDictionary<K, ICollection<V>> errorMap, K errorKey, V errorDesc, params V[] errors)
        {
            if (!errorMap.ContainsKey(errorKey))
            {
                errorMap.Add(errorKey, new List<V>() { errorDesc });
            }

            var errorCollection = errorMap[errorKey];
            foreach (var error in errors)
            {
                errorCollection.Add(error);
            }
        }

        public static void addErrors<K, V>(IDictionary<K, ICollection<V>> errorMap, K errorKey, params V[] errors)
        {
            if (!errorMap.ContainsKey(errorKey))
            {
                errorMap.Add(errorKey, new List<V>());
            }

            var errorCollection = errorMap[errorKey];
            foreach (var error in errors)
            {
                errorCollection.Add(error);
            }
        }

        public static void addErrors<K, V>(IDictionary<K, ICollection<V>> errorMap, K errorKey, IEnumerable<V> errors)
        {
            if (!errorMap.ContainsKey(errorKey))
            {
                errorMap.Add(errorKey, new List<V>());
            }

            var errorCollection = errorMap[errorKey];
            foreach (var error in errors)
            {
                errorCollection.Add(error);
            }
        }
    }
}
