﻿using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Structures.Print
{
    public class ExprRequiresBrackets : Expression.AbstractVisitor<Boolean>
    {
        public ExprType PrevExpr { get; set; }

        public ExprRequiresBrackets(ExprType prevExpr)
        {
            PrevExpr = prevExpr;
        }

        #region Visitor<bool> Members
        
        public override bool visitAnd(AndExpression andExpr)
        {
            if (andExpr.Nexts.Count <= 1)
            {
                return false;
            }

            return PrevExpr == ExprType.Not || PrevExpr == ExprType.Module;
        }

        public override bool visitOr(OrExpression orExpr)
        {
            if (orExpr.Nexts.Count <= 1)
            {
                return false;
            }

            return PrevExpr != ExprType.Or;
        }

        public override bool visitNeg(NegExpression negExpr)
        {
            return PrevExpr == ExprType.Not || PrevExpr == ExprType.Module;
        }

        public override bool visitModule(ModuleExpression moduleExpr)
        {
            return PrevExpr == ExprType.Not && moduleExpr.IsNegative;
        }

        #endregion

        #region AbstractVisitor<bool> Members

        protected override bool defaultValue()
        {
            return false;
        }

        #endregion
    }

    public enum ExprType
    {
        Or,
        And,
        Not,
        Module
    }
}
