﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Structures.Print
{
    class ProgramPrinterWithHidden : Printer
    {
        public ICollection<HiddenData> HiddenData { get; set; }
        private IEnumerator<HiddenData> HiddenDataEnumerator { get; set; }

        public ProgramPrinterWithHidden(ICollection<HiddenData> hiddenData)
        {
            HiddenData = hiddenData;
            HiddenDataEnumerator = hiddenData.Count > 0 ? hiddenData.GetEnumerator() : null;
        }

        public ProgramPrinterWithHidden append(Program program)
        {
            appendHidden(program);
            foreach (var module in program.Modules)
            {
                append(module);
            }
            return this;
        }

        private ProgramPrinterWithHidden append(Module module)
        {
            appendHidden(module);
            Append("module");
            // FIXME: SPACE BETWEEN MODULE AND NAME.
            Append(module.Name);
            return this;
        }

        public void appendHidden(OutputSource source)
        {
            if (source.ParserContext == null)
            {
                return;
            }

            var sourceLine = source.ParserContext.Start.Line;
            var sourceColumn = source.ParserContext.Start.Column;

            while (HiddenDataEnumerator != null)
            {
                var hiddenLine = HiddenDataEnumerator.Current.Token.Line;
                var hiddenColumn = HiddenDataEnumerator.Current.Token.Column;
                
                if (hiddenLine < sourceLine || (hiddenLine == sourceLine && hiddenColumn < sourceColumn))
                {
                    Append(HiddenDataEnumerator.Current.Token.Text);
                    if (!HiddenDataEnumerator.MoveNext())
                    {
                        HiddenDataEnumerator = null;
                        break;
                    }
                } else {
                    break;
                }
            }
        }
    }
}
