﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.Parsers;
using FourqlParser.Structures;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FourqlParserTests
{
    [TestClass]
    public class CorrectModuleTests : BaseModuleTests
    {
        [TestMethod]
        public void KalendarzTest()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("kalendarz.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            prog.Modules.Add(GetSimpleMathModule());
            ParserDispatcher.checkProgram(prog);

            var kalendarz = prog.Modules.Where(module => module.Name.Equals("kalendarz")).First();
            var plany = prog.Modules.Where(module => module.Name.Equals("plany")).First();

            CheckKalendarz(kalendarz);
            CheckPlany(plany);
        }

        private void CheckKalendarz(Module kalendarz)
        {
            Assert.AreEqual(kalendarz.Domains.Count, 2);

            Assert.IsTrue(kalendarz.FindDomain("dzien") != null);
            Assert.AreEqual(kalendarz.FindDomain("dzien").Name, "dzien");
            Assert.AreEqual(kalendarz.FindDomain("dzien").Type, DomainType.INTEGER);

            Assert.IsTrue(kalendarz.FindDomain("miesiac") != null);
            Assert.AreEqual(kalendarz.FindDomain("miesiac").Name, "miesiac");
            Assert.AreEqual(kalendarz.FindDomain("miesiac").Type, DomainType.INTEGER);

            Assert.AreEqual(kalendarz.Relations.Count, 2);

            Assert.IsTrue(kalendarz.FindRelation("swieto") != null);
            Assert.AreEqual(kalendarz.FindRelation("swieto").Name, "swieto");
            Assert.AreEqual(kalendarz.FindRelation("swieto").PrimitiveDomains.Count, 0);
            Assert.AreEqual(kalendarz.FindRelation("swieto").DefinedDomains.Count, 2);
            Assert.AreEqual(kalendarz.FindRelation("swieto").DefinedDomains[0], "dzien");
            Assert.AreEqual(kalendarz.FindRelation("swieto").DefinedDomains[1], "miesiac");

            Assert.IsTrue(kalendarz.FindRelation("okres_swiateczny") != null);
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").Name, "okres_swiateczny");
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").PrimitiveDomains.Count, 0);
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").DefinedDomains.Count, 4);
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").DefinedDomains[0], "dzien");
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").DefinedDomains[1], "miesiac");
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").DefinedDomains[2], "dzien");
            Assert.AreEqual(kalendarz.FindRelation("okres_swiateczny").DefinedDomains[3], "miesiac");

            Assert.AreEqual(kalendarz.Rules.Count, 0);
            Assert.AreEqual(kalendarz.Facts.Count, 3);

            Assert.AreEqual(kalendarz.Facts.ElementAt(0).IsNegative, false);
            Assert.AreEqual(kalendarz.Facts.ElementAt(0).RelationName, "swieto");
            Assert.AreEqual(kalendarz.Facts.ElementAt(0).Value, LogicValue.TRUE);
            Assert.AreEqual(kalendarz.Facts.ElementAt(0).Values[0], new ValueHolder(3, DomainType.INTEGER));
            Assert.AreEqual(kalendarz.Facts.ElementAt(0).Values[1], new ValueHolder(5, DomainType.INTEGER));

            Assert.AreEqual(kalendarz.Facts.ElementAt(1).IsNegative, false);
            Assert.AreEqual(kalendarz.Facts.ElementAt(1).RelationName, "swieto");
            Assert.AreEqual(kalendarz.Facts.ElementAt(1).Value, LogicValue.TRUE);
            Assert.AreEqual(kalendarz.Facts.ElementAt(1).Values[0], new ValueHolder(1, DomainType.INTEGER));
            Assert.AreEqual(kalendarz.Facts.ElementAt(1).Values[1], new ValueHolder(1, DomainType.INTEGER));

            Assert.AreEqual(kalendarz.Facts.ElementAt(2).IsNegative, false);
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).RelationName, "okres_swiateczny");
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).Value, LogicValue.TRUE);
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).Values[0], new ValueHolder(24, DomainType.INTEGER));
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).Values[1], new ValueHolder(12, DomainType.INTEGER));
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).Values[2], new ValueHolder(31, DomainType.INTEGER));
            Assert.AreEqual(kalendarz.Facts.ElementAt(2).Values[3], new ValueHolder(12, DomainType.INTEGER));
        }

        private void CheckPlany(Module plany)
        {
            Assert.AreEqual(plany.Domains.Count, 3);

            Assert.IsTrue(plany.FindDomain("dzien") != null);
            Assert.AreEqual(plany.FindDomain("dzien").Name, "dzien");
            Assert.AreEqual(plany.FindDomain("dzien").Type, DomainType.INTEGER);

            Assert.IsTrue(plany.FindDomain("miesiac") != null);
            Assert.AreEqual(plany.FindDomain("miesiac").Name, "miesiac");
            Assert.AreEqual(plany.FindDomain("miesiac").Type, DomainType.INTEGER);

            Assert.IsTrue(plany.FindDomain("nazwa") != null);
            Assert.AreEqual(plany.FindDomain("nazwa").Name, "nazwa");
            Assert.AreEqual(plany.FindDomain("nazwa").Type, DomainType.LITERAL);

            Assert.AreEqual(plany.Relations.Count, 2);

            Assert.IsTrue(plany.FindRelation("spotkanie") != null);
            Assert.AreEqual(plany.FindRelation("spotkanie").Name, "spotkanie");
            Assert.AreEqual(plany.FindRelation("spotkanie").PrimitiveDomains.Count, 0);
            Assert.AreEqual(plany.FindRelation("spotkanie").DefinedDomains.Count, 3);
            Assert.AreEqual(plany.FindRelation("spotkanie").DefinedDomains[0], "dzien");
            Assert.AreEqual(plany.FindRelation("spotkanie").DefinedDomains[1], "miesiac");
            Assert.AreEqual(plany.FindRelation("spotkanie").DefinedDomains[2], "nazwa");

            Assert.IsTrue(plany.FindRelation("spotkanie_do_odwolania") != null);
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").Name, "spotkanie_do_odwolania");
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").PrimitiveDomains.Count, 0);
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").DefinedDomains.Count, 3);
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").DefinedDomains[0], "dzien");
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").DefinedDomains[1], "miesiac");
            Assert.AreEqual(plany.FindRelation("spotkanie_do_odwolania").DefinedDomains[2], "nazwa");

            Assert.AreEqual(plany.Rules.Count, 2);

            Assert.AreEqual(plany.Rules.ElementAt(0).ToString(),
                new Rule()
                {
                    Head = new Fact()
                    {
                        IsNegative = true,
                        RelationName = "spotkanie",
                        Value = LogicValue.FALSE,
                        Values = new List<ValueHolder>() {
                            new ValueHolder("D", DomainType.VARIABLE),
                            new ValueHolder("M", DomainType.VARIABLE),
                            new ValueHolder("N", DomainType.VARIABLE)
                        }
                    },
                    Body = new AndExpression()
                    {
                        Nexts = new List<Expression>()
                        {
                            new Literal()
                            {
                                RelationName = "spotkanie",
                                Values = new List<ValueHolder>() {
                                    new ValueHolder("D", DomainType.VARIABLE),
                                    new ValueHolder("M", DomainType.VARIABLE),
                                    new ValueHolder("N", DomainType.VARIABLE)
                                }
                            },
                            new Literal()
                            {
                                RelationName = "spotkanie_do_odwolania",
                                Values = new List<ValueHolder>() {
                                    new ValueHolder("D", DomainType.VARIABLE),
                                    new ValueHolder("M", DomainType.VARIABLE),
                                    new ValueHolder("N", DomainType.VARIABLE)
                                }
                            }
                        }
                    }
                }.ToString());

            /*
             * spotkanie_do_odwolania(D, M, N) :-
             *      spotkanie(D, M, N), kalendarz.swieto(D, M)
             *  |
             *      spotkanie(D, M, N), kalendarz.okres_swiateczny(Dp, Mp, Dk, Mk),
             *      math.ge(D, Dp), math.le(D, Dk), math.ge(M, Mp), math.le(M, Mk).
             */
            Assert.AreEqual(plany.Rules.ElementAt(1).ToString(),
                new Rule()
                {
                    Head = new Fact()
                    {
                        IsNegative = false,
                        RelationName = "spotkanie_do_odwolania",
                        Value = LogicValue.TRUE,
                        Values = new List<ValueHolder>() {
                            new ValueHolder("D", DomainType.VARIABLE),
                            new ValueHolder("M", DomainType.VARIABLE),
                            new ValueHolder("N", DomainType.VARIABLE)
                        }
                    },
                    Body = new OrExpression()
                    {
                        Nexts = new List<Expression>()
                        {
                            new AndExpression()
                            {
                                Nexts = new List<Expression>()
                                {
                                    new Literal()
                                    {
                                        RelationName = "spotkanie",
                                        Values = new List<ValueHolder>() {
                                            new ValueHolder("D", DomainType.VARIABLE),
                                            new ValueHolder("M", DomainType.VARIABLE),
                                            new ValueHolder("N", DomainType.VARIABLE)
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "kalendarz",
                                        Next = new Literal()
                                        {
                                            RelationName = "swieto",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("D", DomainType.VARIABLE),
                                                new ValueHolder("M", DomainType.VARIABLE)
                                            }
                                        }
                                    }
                                }
                            },
                            new AndExpression()
                            {
                                Nexts = new List<Expression>()
                                {
                                    new Literal()
                                    {
                                        RelationName = "spotkanie",
                                        Values = new List<ValueHolder>() {
                                            new ValueHolder("D", DomainType.VARIABLE),
                                            new ValueHolder("M", DomainType.VARIABLE),
                                            new ValueHolder("N", DomainType.VARIABLE)
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "kalendarz",
                                        Next = new Literal()
                                        {
                                            RelationName = "okres_swiateczny",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("Dp", DomainType.VARIABLE),
                                                new ValueHolder("Mp", DomainType.VARIABLE),
                                                new ValueHolder("Dk", DomainType.VARIABLE),
                                                new ValueHolder("Mk", DomainType.VARIABLE)
                                            }
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "math",
                                        Next = new Literal()
                                        {
                                            RelationName = "ge",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("D", DomainType.VARIABLE),
                                                new ValueHolder("Dp", DomainType.VARIABLE)
                                            }
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "math",
                                        Next = new Literal()
                                        {
                                            RelationName = "le",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("D", DomainType.VARIABLE),
                                                new ValueHolder("Dk", DomainType.VARIABLE)
                                            }
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "math",
                                        Next = new Literal()
                                        {
                                            RelationName = "ge",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("M", DomainType.VARIABLE),
                                                new ValueHolder("Mp", DomainType.VARIABLE)
                                            }
                                        }
                                    },
                                    new ModuleExpression()
                                    {
                                        IsNegative = false,
                                        ModuleName = "math",
                                        Next = new Literal()
                                        {
                                            RelationName = "le",
                                            Values = new List<ValueHolder>() {
                                                new ValueHolder("M", DomainType.VARIABLE),
                                                new ValueHolder("Mk", DomainType.VARIABLE)
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }.ToString());

            Assert.AreEqual(plany.Facts.Count, 3);

            Assert.AreEqual(plany.Facts.ElementAt(0).IsNegative, false);
            Assert.AreEqual(plany.Facts.ElementAt(0).RelationName, "spotkanie");
            Assert.AreEqual(plany.Facts.ElementAt(0).Value, LogicValue.TRUE);
            Assert.AreEqual(plany.Facts.ElementAt(0).Values[0], new ValueHolder(1, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(0).Values[1], new ValueHolder(1, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(0).Values[2], new ValueHolder("mariot", DomainType.LITERAL));

            Assert.AreEqual(plany.Facts.ElementAt(1).IsNegative, false);
            Assert.AreEqual(plany.Facts.ElementAt(1).RelationName, "spotkanie");
            Assert.AreEqual(plany.Facts.ElementAt(1).Value, LogicValue.TRUE);
            Assert.AreEqual(plany.Facts.ElementAt(1).Values[0], new ValueHolder(26, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(1).Values[1], new ValueHolder(12, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(1).Values[2], new ValueHolder("stadion_narodowy", DomainType.LITERAL));

            Assert.AreEqual(plany.Facts.ElementAt(2).IsNegative, false);
            Assert.AreEqual(plany.Facts.ElementAt(2).RelationName, "spotkanie");
            Assert.AreEqual(plany.Facts.ElementAt(2).Value, LogicValue.TRUE);
            Assert.AreEqual(plany.Facts.ElementAt(2).Values[0], new ValueHolder(5, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(2).Values[1], new ValueHolder(5, DomainType.INTEGER));
            Assert.AreEqual(plany.Facts.ElementAt(2).Values[2], new ValueHolder("dom", DomainType.LITERAL));
        }

        private Module GetSimpleMathModule()
        {
            var math = new Module() { IsInternalExternal = true, Name = "math" };
            math.Relations.Add(new Relation() { Name = "le", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            math.Relations.Add(new Relation() { Name = "ge", PrimitiveDomains = new Dictionary<int, DomainType>() { { 0, DomainType.INTEGER }, { 1, DomainType.INTEGER } } });
            return math;
        }
    }
}
