﻿using System;
using System.IO;
using FourqlParser.Parsers;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FourqlParserTests
{
    [TestClass]
    public class DefinitionErrorTests : BaseModuleTests
    {
        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated module 'duplicatedModule'.")]
        public void DuplicatedModulesNotAllowed1()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-module.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated module 'duplicatedModuleUsed'.")]
        public void DuplicatedModulesNotAllowed2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-module_2.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated definition of domain 'dom' in module 'duplicatedDefinedDomain'.")]
        public void DuplicatedDomainsNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-domain.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated definition of relation 'r' in module 'duplicatedRelation'.")]
        public void DuplicatedRelationNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-relation.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Found cycles (Strongly Connected Components) that consist of modules:\n\t0. cycleModule2, cycleModule1")]
        public void ModuleCycleNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "cycle-module.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'Y' is defined by two or more primitive types - literal in r(X, Y) (param no. 1) from module 'headAndBodyVariableDifferInAlternativeRule' and integer in w(Y) (param no. 0) from module 'headAndBodyVariableDifferInAlternativeRule'")]
        public void HeadAndBodyVariablesDifferInAlternatives1()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardifferinalternativebody-rule_1.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'X' is defined by two or more primitive types - literal in rel(X) (param no. 0) from module 'headAndBodyVariableDifferInAlternativesRule' and date in rel2(X) (param no. 0) from module 'headAndBodyVariableDifferInAlternativesRule'")]
        public void HeadAndBodyVariablesDifferInAlternatives2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardifferinalternativebody-rule_2.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(X, X)' in module 'headAndBodyVariableDifferRule' - fact uses two different primitive types (previously literal, now integer) for variable 'X'.")]
        public void HeadAndBodyVariablesDiffer()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardiffer-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "There is no variable 'Y' in the alternative: r(X, X) of the body of the rule: r(X, Y) :- r(X, X).")]
        public void NoVariableFromRuleHeadInBody()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "novariableinbody-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "There is no variable 'Y' in the alternative: r(X, X) of the body of the rule: r(X, Y) :- r(X, Y) | r(X, X).")]
        public void NoVariableFromRuleHeadInAlternative()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "novariableinalternativebody-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(1)' in module 'noRelationRule' - relation 'r' does not exist in the module.")]
        public void NoRelationForRule()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "norelation-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r()' in module 'noRelationFact' - relation 'r' does not exist in the module.")]
        public void NoRelationForFact()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "norelation-fact.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of relation 'r' in module 'noDomainRelation' - domain 'domain' does not exist in the module.")]
        public void NoDomainForRelation()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "nodomain-relation.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Module 'selfreference' used in expression 'selfreference.rel(X)' is non-existent.")]
        public void ModuleSelfReferenceInRule()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "moduleselfreference-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'Y' is defined by two or more primitive types - string in rel_string(Y) (param no. 0) from module 'localVariablesDifferRule' and integer in rel_integer(Y) (param no. 0) from module 'localVariablesDifferRule'")]
        public void LocalVariablesDifferInAlternatives()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "localvariablesdiffer-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(X, X)' in module 'multiplePrimitiveTypeVariableLiteral' - fact uses two different primitive types (previously literal, now integer) for variable 'X'.")]
        public void MultiPrimitiveTypeVariableNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "multiprimitivetypevariable-literal.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r()' in module 'invalidParamCountFact' - base relation 'r' has 1 arguments, but the literal has 0.")]
        public void FactCannotBeDifferentThanRelationOnParamCount()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalidparamcount-fact.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(a, b)' in module 'invalidParamCountTooManyRule' - base relation 'r' has 1 arguments, but the literal has 2.")]
        public void RuleHeadCannotBeDifferentThanRelationOnParamCount()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalidparamcount-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }
    }
}
