﻿using FourqlParser.Structures;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FourqlParser.Parsers.Converters
{
    class ModuleConverter : ModuleParserBaseVisitor<Module>
    {
        public Module Module { get; private set; }

        public override Module VisitModule(ModuleParser.ModuleContext context)
        {
            Module = new Module() { Name = context.ModuleName.Text, ParserContext = context };
            // FIXME: Should walk through known selected nodes.
            VisitChildren(context);
            addPrimitiveDomainRelations(Module);
            return Module;
        }

        private void addPrimitiveDomainRelations(Module module)
        {
            foreach (DomainType type in Enum.GetValues(typeof(DomainType)))
            {
                if (!type.Equals(DomainType.VARIABLE))
                {
                    var domainRelation = new Relation() { Name = type.ToString().ToLower() };
                    domainRelation.PrimitiveDomains[0] = type;
                    module.DomainRelations.Add(domainRelation);
                }
            }
        }

        public override Module VisitNoDomains(ModuleParser.NoDomainsContext context)
        {
            return null;
        }

        public override Module VisitDomain(ModuleParser.DomainContext context)
        {
            Domain domain = context.Accept(new DomainConverter());
            Module.Domains.Add(domain);
            var domainRelation = new Relation() {Name = domain.Name};
            domainRelation.DefinedDomains[0] = domain.Name;
            Module.DomainRelations.Add(domainRelation);
            return null;
        }

            /*
             * var domainRule = new Rule()
            {
                Head = new Fact()
                {
                    RelationName = domainRelation.Name,
                    Values = new List<ValueHolder>() {new ValueHolder("X", DomainType.VARIABLE)}
                },
                Body = new ModuleExpression()
                {
                    ModuleName = domain.ImportedFromModule,
                    Next =
                        new Literal()
                        {
                            RelationName = domain.Name,
                            Values = new List<ValueHolder>() {new ValueHolder("X", DomainType.VARIABLE)}
                        }
                },
                DomainRule = true
            };
            Module.DomainRules.Add(domainRule); */

        public override Module VisitRelation(ModuleParser.RelationContext context)
        {
            var relation = context.Accept(new RelationConverter());
            Module.Relations.Add(relation);
            return null;
        }

        public override Module VisitRuleDef(ModuleParser.RuleDefContext context)
        {
            var rule = context.Accept(new RuleConverter() {ModuleName = Module.Name});
            Module.Rules.Add(rule);
            return null;
        }

        public override Module VisitFact(ModuleParser.FactContext context)
        {
            Module.Facts.Add(context.Accept(new FactConverter()));
            return null;
        }
    }
}
