parser grammar ModuleParser;

options {
	tokenVocab = FourqlLexer;
}



/*
 * Parser Rules
 */

queryRule 
	:	Query=expressionRule (DOT | EOF | SEMICOLON)		
	;

programRule
	: Modules=moduleRule* EOF								# Program
	;

moduleRule
	: MODULE ModuleName=IDENT OPENING_CURLY Domains=domainsRule Relations=relationsRule Rules=rulesRule Facts=factsRule CLOSING_CURLY # Module
	;
	
// Domains:
domainsRule
	: DOMAINS OPENING_CURLY DomainList=domainRule* CLOSING_CURLY				# Domains
	| /* Empty */																# NoDomains
	;

domainRule
	: DomainName=IDENT COLON Type=typeRule domainImports? SEMICOLON							# Domain
	;

domainImports
	: OPENING_CURLY IMPORT VALUES? FROM domainName (COMMA domainName)* CLOSING_CURLY
	;

domainName
	: ModuleName=IDENT DOT DomainName=IDENT
	;

// Relations:
relationsRule
	: RELATIONS OPENING_CURLY RelationList=relationRule* CLOSING_CURLY			# Relations
	| /* Empty */																# NoRelations
	;

relationRule
	: RelationName=IDENT OPENING_PARENT RelationArgs=relationArgsRule CLOSING_PARENT SEMICOLON # Relation
	;

relationArgsRule
	: Head=relationArgRule (COMMA Tail=relationArgsRule)?	# RelationArguments
	| /* Empty */											# NoRelationArguments
	;

relationArgRule
	: DomainName=IDENT										# RelationDomainArg
	| Type=typeRule											# RelationTypeArg
	;

// Rules:
rulesRule
	: RULES OPENING_CURLY RuleList=ruleRule* CLOSING_CURLY	# Rules
	| /* Empty */											# NoRules
	;

ruleRule
	: Head=ruleHeadRule IS_CONCLUSION_OF Body=expressionRule SEMICOLON	# RuleDef
	;

ruleHeadRule
	: (IsNegative=NOT)? Literal=literalRule						# RuleHead
	;

// expressions 
expressionRule
	: Expr=alternativeRule											# Expr
	;

alternativeRule
	: Head=conjunctionRule (OR Tail=alternativeRule)?				# OrExpr
	;

conjunctionRule
	: Head=negExprRule (AND Tail=conjunctionRule)?					# AndExpr
	;

negExprRule
	: NOT Expr=unaryExprRule										# NegExpr
	| Expr=unaryExprRule											# NonNegExpr
	;

unaryExprRule
	: OPENING_PARENT Expr=alternativeRule CLOSING_PARENT															# ParentExpr
	| ModuleName=IDENT DOT Expr=unaryExprRule (IN OPENING_CURLY InList=inLogicArgsRule CLOSING_CURLY)? # ModuleExpr
	| Literal=literalRule																							# LiteralExpr
	| EXISTS VariableName=VARIABLE COLON (ModuleName=IDENT DOT)? (DomainName=IDENT | PrimitiveDomain=typeRule) Expr=negExprRule 		# ExistsExpr
	| ALL VariableName=VARIABLE COLON (ModuleName=IDENT DOT)? (DomainName=IDENT | PrimitiveDomain=typeRule) Expr=negExprRule 			# AllExpr
	;

inLogicArgsRule
	: Head=logicValueRule											# SingleInLogicArgument
	| Head=logicValueRule COMMA Tail=inLogicArgsRule				# InLogicArguments
	;

literalRule
	: (RelationName=IDENT | DomainRelationName=typeRule) OPENING_PARENT LiteralArgs=literalArgsRule CLOSING_PARENT # Literal
	;

literalArgsRule
	: Head=literalArgRule (COMMA Tail=literalArgsRule)?				# LiteralArguments
	| /* Empty */													# NoLiteralArguments
	;

literalArgRule
	: VariableName=VARIABLE											# LiteralVariableArgument
	| Value=valueRule												# LiteralValueArgument
	;


// Facts:
factsRule
	: FACTS OPENING_CURLY FactList=factRule*	CLOSING_CURLY		# Facts
	| /* Empty */													# NoFacts
	;

factRule
	: (IsNegative=NOT)? RelationName=IDENT OPENING_PARENT FactArgs=factArgsRule CLOSING_PARENT SEMICOLON # Fact
	;

factArgsRule
	: Head=valueRule (COMMA Tail=factArgsRule)?	# FactArguments
	| /* Empty */								# NoFactArguments
	;

// Values - used in facts and rule literals.
valueRule
	: LogicValue=logicValueRule					# ValueLogic
	| LiteralValue=IDENT						# ValueLiteral
	| StringValue=VALUE_STRING					# ValueString
	| IntegerValue=VALUE_INTEGER				# ValueInteger
	| RealValue=VALUE_REAL						# ValueReal
	| DateValue=VALUE_DATE						# ValueDate
	| DateTimeValue=VALUE_DATETIME				# ValueDateTime
	;

logicValueRule
	: VALUE_TRUE								# ValueTrue
	| VALUE_FALSE								# ValueFalse
	| VALUE_INCONS								# ValueInconsistent
	| VALUE_UNKNOWN								# ValueUnknown
	;

// Types - used in relation and domain definitions.
typeRule
	: TYPE_LOGIC								# TypeLogic
	| TYPE_LITERAL								# TypeLiteral
	| TYPE_STRING								# TypeString
	| TYPE_INTEGER								# TypeInteger
	| TYPE_REAL									# TypeReal
	| TYPE_DATE									# TypeDate
	| TYPE_DATETIME								# TypeDateTime
	;