﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FourqlParser.StaticAnalysis;

namespace FourqlParser.Structures
{
    public class Module : OutputSource
    {
        public String Name { get; set; }
        public bool IsInternalExternal { get; set; }
        public ICollection<Domain> Domains { get; set; }
        public ICollection<Relation> Relations { get; set; }

        public ICollection<Relation> DomainRelations { get; set; } 
        public ICollection<Rule> Rules { get; set; }
        public ICollection<Fact> Facts { get; set; }
        public ModuleInvalidDefinitions InvalidDefinitions { get; set; }
        public ICollection<Rule> DomainRules { get; set; }

        public Module()
        {
            Domains = new List<Domain>();
            Relations = new List<Relation>();
            Rules = new List<Rule>();
            Facts = new List<Fact>();
            InvalidDefinitions = new ModuleInvalidDefinitions();
            DomainRelations = new List<Relation>();
            DomainRules = new List<Rule>();
        }

        public Domain FindDomain(string domainName)
        {
            return (from dom in Domains
                where dom.Name.Equals(domainName)
                select dom).FirstOrDefault();
        }

        public Relation FindRelation(string relationName)
        {
            return (from rel in Relations.Concat(DomainRelations)
                    where rel.Name.Equals(relationName) 
                    select rel)
                    .FirstOrDefault();
            
        }


        public ICollection<Rule> FindRules(string relationName)
        {
            var rules = new List<Rule>();
            foreach (var rule in Rules)
            {
                if (rule.Head.RelationName.Equals(relationName))
                {
                    rules.Add(rule);
                }
            }
            return rules;
        }
    }
}
