﻿using System;
using System.IO;
using FourqlParser.Parsers;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FourqlParserTests
{
    [TestClass]
    public class DefinitionErrorTests : BaseModuleTests
    {

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof (System.Exception),
            "Relation cannot have the same name as domain! Relation 'dom' in module 'm'.")]
        public void SameRelationAndDomainNameNotAllowed()
        {

            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicateDomainRelationName.4ql"), errors, null);
            Console.WriteLine(errors.ParserErrors.Count);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception),
            "Invalid definition of literal 'dom(a)' in module 'domainRuleHeadInvalid' - cannot use domain 'dom' as a relation in this context.")]
        public void DomainAsRelationNotAllowedInRuleHead()
        {

            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "domainRuleHead.4ql"), errors, null);
            Console.WriteLine(errors.ParserErrors.Count);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception),
            "Invalid definition of literal 'dom(a)' in module 'domainFactInvalid' - cannot use domain 'dom' as a relation in this context.")]
        public void DomainAsRelationNotAllowedInFacts()
        {

            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "domainFactInvalid.4ql"), errors, null);
            Console.WriteLine(errors.ParserErrors.Count);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated module 'duplicatedModule'.")]
        public void DuplicatedModulesNotAllowed1()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-module.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated module 'duplicatedModuleUsed'.")]
        public void DuplicatedModulesNotAllowed2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-module_2.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated definition of domain 'dom' in module 'duplicatedDefinedDomain'.")]
        public void DuplicatedDomainsNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-domain.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Duplicated definition of relation 'r' in module 'duplicatedRelation'.")]
        public void DuplicatedRelationNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "duplicated-relation.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Found cycles (Strongly Connected Components) that consist of modules:\n\t0. cycleModule2, cycleModule1")]
        public void ModuleCycleNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "cycle-module.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'Y' is defined by two or more primitive types - literal in r(X, Y) (param no. 1) from module 'headAndBodyVariableDifferInAlternativeRule' and integer in w(Y) (param no. 0) from module 'headAndBodyVariableDifferInAlternativeRule'")]
        public void HeadAndBodyVariablesDifferInAlternatives1()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardifferinalternativebody-rule_1.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'X' is defined by two or more primitive types - literal in rel(X) (param no. 0) from module 'headAndBodyVariableDifferInAlternativesRule' and date in rel2(X) (param no. 0) from module 'headAndBodyVariableDifferInAlternativesRule'")]
        public void HeadAndBodyVariablesDifferInAlternatives2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardifferinalternativebody-rule_2.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(X, X)' in module 'headAndBodyVariableDifferRule' - fact uses two different primitive types (previously literal, now integer) for variable 'X'.")]
        public void HeadAndBodyVariablesDiffer()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "headandbodyvardiffer-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "There is no variable 'Y' in the body of the rule: r(X, Y) when r(X, X);")]
        public void NoVariableFromRuleHeadInBody()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "novariableinbody-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

  

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(1)' in module 'noRelationRule' - relation 'r' does not exist in the module.")]
        public void NoRelationForRule()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "norelation-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r()' in module 'noRelationFact' - relation 'r' does not exist in the module.")]
        public void NoRelationForFact()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "norelation-fact.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of relation 'r' in module 'noDomainRelation' - domain 'domain' does not exist in the module.")]
        public void NoDomainForRelation()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "nodomain-relation.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Module 'selfreference' used in expression 'selfreference.rel(X)' is non-existent.")]
        public void ModuleSelfReferenceInRule()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "moduleselfreference-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Variable 'Y' is defined by two or more primitive types - string in rel_string(Y) (param no. 0) from module 'localVariablesDifferRule' and integer in rel_integer(Y) (param no. 0) from module 'localVariablesDifferRule'")]
        public void LocalVariablesDifferInAlternatives()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "localvariablesdiffer-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(X, X)' in module 'multiplePrimitiveTypeVariableLiteral' - fact uses two different primitive types (previously literal, now integer) for variable 'X'.")]
        public void MultiPrimitiveTypeVariableNotAllowed()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "multiprimitivetypevariable-literal.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r()' in module 'invalidParamCountFact' - base relation 'r' has 1 arguments, but the literal has 0.")]
        public void FactCannotBeDifferentThanRelationOnParamCount()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalidparamcount-fact.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Invalid definition of literal 'r(a, b)' in module 'invalidParamCountTooManyRule' - base relation 'r' has 1 arguments, but the literal has 2.")]
        public void RuleHeadCannotBeDifferentThanRelationOnParamCount()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalidparamcount-rule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Bound variable 'Y' is defined as type literal but in 'r(Y)' (param no. 0) from module 'm' it has type integer")]
        public void BoundVarWrongType()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "boundVarDifferentType.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Bound variable 'X' from rule 'r(X, Y) when EXISTS X : l r(Y, X);' cannot appear in the head of the rule!")]
        public void DuplicateBoundVar()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "boundVariableSameName.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }


        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Imported domain 'dom2' doesn't exist in module 'n'.")]
        public void ErrorImportedDomain()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalid-domain-import.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Type mismatch: imported domain 'n.dom1' has type INTEGER, expected LITERAL.")]
        public void ErrorImportedDomain2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalid-domain-import-2.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }
        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Module 's' from definition of domain 'dom1' in module 'm' doesn't exist.")]
        public void ErrorImportedDomain3()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "invalid-domain-import-3.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }

        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Domain 'dom' used in expression 'ALL X : n.dom r(Y, X)' is non-existent in module quantifierNoModule.")]
        public void ErrorQuantifierDomain()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "quantifierNoDomainInModule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }


        [TestMethod]
        [ExpectedExceptionWithMessageAttribute(typeof(System.Exception), "Module 'n' of domain literal used in expression 'EXISTS X : n.literal r(Y, X)' is non-existent.")]
        public void ErrorQuantifierDomain2()
        {
            var errors = new ParserDispatcher.Errors();
            var progContext = ParserDispatcher.parseProgram(GetTestContent("Errors", "quantifierNoModule.4ql"), errors, null);
            Assert.IsTrue(errors.ParserErrors.Count == 0);
            var prog = ParserDispatcher.toProgram(progContext);
            ParserDispatcher.checkProgram(prog);
        }
    }
}
