﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FourqlParserTests
{
    // Taken from: http://stackoverflow.com/questions/4710129/how-do-i-enforce-exception-message-with-expectedexception-attribute
    // This version can consider inner exception messages.
    public class ExpectedExceptionWithMessageAttribute : ExpectedExceptionBaseAttribute
    {
        public Type ExceptionType { get; set; }
        public string ExpectedMessage { get; set; }
        public bool CompareMessageFromOuterException { get; set; }

        public ExpectedExceptionWithMessageAttribute(Type exceptionType)
        {
            this.ExceptionType = exceptionType;
        }

        public ExpectedExceptionWithMessageAttribute(Type exceptionType, string expectedMessage)
        {
            this.ExceptionType = exceptionType;
            this.ExpectedMessage = expectedMessage;
        }

        public ExpectedExceptionWithMessageAttribute(Type exceptionType, string expectedMessage, bool compareMessageFromOuterException)
        {
            this.ExceptionType = exceptionType;
            this.ExpectedMessage = expectedMessage;
            this.CompareMessageFromOuterException = compareMessageFromOuterException;
        }

        protected override void Verify(Exception e)
        {
            if (e.GetType() != this.ExceptionType)
            {
                Assert.Fail(String.Format(
                                "ExpectedExceptionWithMessageAttribute failed. Expected exception type: {0}. Actual exception type: {1}. Exception message: {2}",
                                this.ExceptionType.FullName,
                                e.GetType().FullName,
                                e.Message
                                )
                            );
            }

            while (!CompareMessageFromOuterException && e.InnerException != null)
            {
                e = e.InnerException;
            }

            var actualMessage = e.Message.Trim();
            if (this.ExpectedMessage != null)
            {
                Assert.AreEqual(this.ExpectedMessage, actualMessage);
            }

            Console.Write("ExpectedExceptionWithMessageAttribute:" + e.Message);
        }
    }
}
